package com.tofly.yswzMaterials.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.yswzMaterials.entity.jindie.Material;
import com.tofly.yswzMaterials.service.jindie.DockingService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

/**
 * @author Bubble
 */
@RestController
@AllArgsConstructor
@RequestMapping("/docking")
@Api(tags = "金蝶-基础信息接口")
public class DockingController {

    private final DockingService dockingService;

    @GetMapping("/basicInfoQueryAll")
    @ApiOperation("金蝶-查询全部基础消息接口")
    @ApiImplicitParam(name = "urlCode",value = "0 物资 1 仓库 2 供应商 3 部门 4 员工 5 计量单位", required = true,dataType = "string")
    public ResultRespone<List<Material>> basicInfoQueryAll(String urlCode) {
        return ResultRespone.success(dockingService.basicInfoQueryAll(urlCode));
    }

    @GetMapping("/basicInfoQuery")
    @ApiOperation("金蝶-基础消息接口操作")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "urlCode",value = "0 物资 1 仓库 2 供应商 3 部门 4 员工 5 计量单位", required = true,dataType = "string"),
            @ApiImplicitParam(name = "type",value = "0 查询 1 新增 2 删除", required = true,dataType = "string")
    })

    public ResultRespone<Map> basicInfoQuery(Material material, String urlCode,String type) {
        return ResultRespone.success(dockingService.basicInfoQuery(material,urlCode,type));
    }

    @PostMapping("/syncBasicInfo")
    @ApiOperation("金蝶-基础信息同步")
    @ApiImplicitParam(name = "type",value = "0 物资 1 仓库 2 供应商 3 部门 4 员工 5 计量单位", required = true,dataType = "string")
    public ResultRespone<Boolean> syncBasicInfo(String type) {
        return ResultRespone.success(dockingService.syncBasicInfo(type));
    }

    @GetMapping("/getDeviceUnitGroup")
    @ApiOperation("金蝶-获取物料单位组")
    public ResultRespone getDeviceUnitGroup(){
        return ResultRespone.success(dockingService.getDeviceUnitGroup());
    }

    @GetMapping("/getVendorList")
    @ApiOperation("金蝶-获取厂家列表")
    public ResultRespone getVendorList(Page page){
        return ResultRespone.success(dockingService.getVendorList(page));
    }

    @GetMapping("/getDeptList")
    @ApiOperation("金蝶-获取部门列表")
    public ResultRespone getDeptList(Page page){
        return ResultRespone.success(dockingService.getDeptList(page));
    }

    @GetMapping("/getEmployeeList")
    @ApiOperation("金蝶-获取人员列表")
    public ResultRespone getEmployeeList(Page page){
        return ResultRespone.success(dockingService.getEmployeeList(page));
    }
}
