package com.tofly.yswzMaterials.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.yswzMaterials.entity.Picking;
import com.tofly.yswzMaterials.entity.Storage;
import com.tofly.yswzMaterials.entity.dto.CommonQueryDto;
import com.tofly.yswzMaterials.entity.vo.PickingParamVo;
import com.tofly.yswzMaterials.service.PickingService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * @author Bubble
 */
@RestController
@AllArgsConstructor
@RequestMapping("/picking")
@Api(tags = "金蝶-生产领料接口")
public class PickingController {

    private final PickingService pickingService;


    @GetMapping("/getPickingInfo")
    @ApiOperation("获取生产领料单")
    public ResultRespone<PickingParamVo> getPickingInfo(String code) {
        return ResultRespone.success(pickingService.getPickingInfo(code));
    }

    @PostMapping("/insertPicking")
    @ApiOperation("添加生产领料单")
    public ResultRespone<Boolean> insertPicking(@Validated Picking picking,
                                                  @RequestPart(required = false) @ApiParam("附件列表") List<MultipartFile> file){
        return ResultRespone.success(pickingService.insertPicking(picking,file));
    }

    @PutMapping("/updatePicking")
    @ApiOperation("修改生产领料单")
    public ResultRespone<Boolean> updatePicking(@Validated Picking picking,
                                                  @RequestPart(required = false) @ApiParam("附件列表") List<MultipartFile> file){
        return ResultRespone.success(pickingService.updatePicking(picking,file));
    }

    @GetMapping("/getById")
    @ApiModelProperty("根据id查询详情")
    public ResultRespone<Picking> getById(Long id) {
        return ResultRespone.success(pickingService.getById(id));
    }


    @GetMapping("/page")
    @ApiOperation("分页查询")
    public ResultRespone<Page<Picking>> page(Page page, CommonQueryDto dto) {
        return ResultRespone.success(pickingService.getPageInfo(page, dto));
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除")
    public ResultRespone<Boolean> delete(String ids){
        return ResultRespone.success(pickingService.delete(ids));
    }
}
