package com.tofly.yswzMaterials.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.yswzMaterials.entity.Profit;
import com.tofly.yswzMaterials.entity.dto.CommonQueryDto;
import com.tofly.yswzMaterials.entity.vo.ProfitParamVo;
import com.tofly.yswzMaterials.service.ProfitService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * @author Bubble
 */
@RestController
@AllArgsConstructor
@RequestMapping("/profit")
@Api(tags = "金蝶-盘盈接口")
public class ProfitController {

    private final ProfitService profitService;

    @GetMapping("/getProfitInfo")
    @ApiOperation("获取盘盈")
    public ResultRespone<ProfitParamVo> getProfitInfo(String code) {
        return ResultRespone.success(profitService.getProfitInfo(code));
    }

    @PostMapping("/insertProfit")
    @ApiOperation("增加盘盈")
    public ResultRespone<Boolean> insertProfit(@Validated Profit profit, @RequestPart(required = false) @ApiParam("附件列表")List<MultipartFile> file) {
        return ResultRespone.success(profitService.insertProfit(profit,file));
    }

    @PutMapping("/updateProfit")
    @ApiOperation("修改盘盈")
    public ResultRespone<Boolean> updateProfit(@Validated Profit profit, @RequestPart(required = false) @ApiParam("附件列表")List<MultipartFile> file) {
        return ResultRespone.success(profitService.updateProfit(profit,file));
    }

    @GetMapping("/page")
    @ApiOperation("分页查询")
    public ResultRespone<Page<Profit>> getPageInfo(Page page, CommonQueryDto dto) {
        return ResultRespone.success(profitService.getPageInfo(page,dto));
    }

    @DeleteMapping("/deleteProfit")
    @ApiOperation("删除盘盈")
    public ResultRespone<Boolean> deleteProfit(String ids) {
        return ResultRespone.success(profitService.deleteProfit(ids));
    }

    @GetMapping("/getById")
    @ApiOperation("id查询详情")
    public ResultRespone<Profit> getProfitById(Long id) {
        return ResultRespone.success(profitService.getProfitById(id));
    }

}
