/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.yswzMaterials.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.yswzMaterials.entity.StocksDeviceInfo;
import com.tofly.yswzMaterials.entity.dto.DeviceInfoQuery;
import com.tofly.yswzMaterials.entity.dto.DeviceInfoStoreQuery;
import com.tofly.yswzMaterials.service.StocksDeviceInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * 物资设备信息表（包含物料、设备、药品）
 *
 * @author JF
 * @date Wed Mar 29 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfstocksdeviceinfo")
@Api(tags = "物资设备信息表（包含物料、设备、药品）接口")
public class StocksDeviceInfoController {

  private final StocksDeviceInfoService tfStocksDeviceInfoService;

  /**
   * 分页查询
   *
   * @param page 分页对象
   * @param query 物资设备信息表（包含物料、设备、药品）
   * @return
   */
  @GetMapping("/page")
  @ApiOperation(value = "分页查询")
  public ResultRespone getDeviceInfoPage(Page page, @ModelAttribute DeviceInfoQuery query) {
    return ResultRespone.success(tfStocksDeviceInfoService.getDataPage(page, query));
  }

  @GetMapping("/devicestorelist")
  @ApiOperation(value = "出库/使用记录时候，选择对应的物资列表")
  public ResultRespone getDeviceStorelistPage(
      Page page, @Validated DeviceInfoStoreQuery deviceInfoStoreQuery) {
    return ResultRespone.success(
        tfStocksDeviceInfoService.getDeviceStorelistPage(page, deviceInfoStoreQuery));
  }

  /**
   * 通过id查询物资设备信息表（包含物料、设备、药品）
   *
   * @param id id
   * @return ResultRespone
   */
  @GetMapping("/{id}")
  @ApiOperation(value = "通过ID查询")
  @ApiImplicitParams({
    @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
  })
  public ResultRespone getById(@PathVariable("id") Long id) {
    return ResultRespone.success(tfStocksDeviceInfoService.getById(id));
  }

  /**
   * 新增物资设备信息表（包含物料、设备、药品）
   *
   * @param deviceInfo 物资设备信息表（包含物料、设备、药品）
   * @return ResultRespone
   */
  @ToFlyAppLog(title = "新增物资设备信息表（包含物料、设备、药品）")
  @ApiOperation(value = "新增物资设备信息表（包含物料、设备、药品）")
  @PostMapping
  public ResultRespone save(@RequestBody @Validated StocksDeviceInfo deviceInfo) {
    return ResultRespone.success(tfStocksDeviceInfoService.saveDeviceInfo(deviceInfo));
  }

  /**
   * 修改物资设备信息表（包含物料、设备、药品）
   *
   * @param deviceInfo 物资设备信息表（包含物料、设备、药品）
   * @return ResultRespone
   */
  @ToFlyAppLog(title = "修改物资设备信息表（包含物料、设备、药品）")
  @ApiOperation(value = "修改物资设备信息表（包含物料、设备、药品）")
  @PutMapping
  public ResultRespone updateById(@RequestBody @Validated StocksDeviceInfo deviceInfo) {
    return ResultRespone.success(tfStocksDeviceInfoService.editDeviceInfo(deviceInfo));
  }

  /**
   * 通过id批量删除物资设备信息表（包含物料、设备、药品）
   *
   * @param ids id
   * @return ResultRespone
   */
  @ToFlyAppLog(title = "通过id批量删除物资设备信息表（包含物料、设备、药品）")
  @ApiOperation(value = "通过ID批量删除物资设备信息表（包含物料、设备、药品）")
  @ApiImplicitParams({
    @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
  })
  @DeleteMapping("/deleteByIds")
  public ResultRespone removeById(String ids) {
    return ResultRespone.success(tfStocksDeviceInfoService.deleteByIds(ids));
  }


  /**
   * 分页查询
   *
   * @param page 分页对象
   * @param deviceId 设备台账，详情数据，包含维修信息，保养信息，巡检信息
   * @return
   */
  @GetMapping("/repairList")
  @ApiOperation(value = "设备维修信息，分页查询")
  @ApiImplicitParam(name = "deviceId", value = "设备ID")
  public ResultRespone getDeviceRepairListPage(
      Page page, @RequestParam(value = "deviceId") Long deviceId) {
    return ResultRespone.success(tfStocksDeviceInfoService.getDeviceRepairListPage(page, deviceId));
  }

  /**
   * 分页查询
   *
   * @param page 分页对象
   * @param deviceId 设备台账，详情数据，包含维修信息，保养信息，巡检信息
   * @return
   */
  @GetMapping("/maintainList")
  @ApiOperation(value = "设备保养信息，分页查询")
  @ApiImplicitParam(name = "deviceId", value = "设备ID")
  public ResultRespone getDeviceMaintainListPage(
      Page page, @RequestParam(value = "deviceId") Long deviceId) {
    return ResultRespone.success(
        tfStocksDeviceInfoService.getDeviceMaintainListPage(page, deviceId));
  }

  /**
   * 分页查询
   *
   * @param page 分页对象
   * @param deviceId 设备台账，详情数据，包含维修信息，保养信息，巡检信息
   * @return
   */
  @GetMapping("/patrolList")
  @ApiOperation(value = "设备巡检信息，分页查询")
  @ApiImplicitParam(name = "deviceId", value = "设备ID")
  public ResultRespone getDevicePatrolListPage(
      Page page, @RequestParam(value = "deviceId") Long deviceId) {
    return ResultRespone.success(tfStocksDeviceInfoService.getDevicePatrolListPage(page, deviceId));
  }

  /**
   * 分页查询
   *
   * @param deviceId 设备台账，详情数据，包含维修信息，保养信息，巡检信息
   * @return
   */
  @GetMapping("/report")
  @ApiOperation(value = "设备图表统计")
  @ApiImplicitParam(name = "deviceId", value = "设备ID")
  public ResultRespone getDeviceReport(@RequestParam(value = "deviceId") Long deviceId) {
    return ResultRespone.success(tfStocksDeviceInfoService.getDeviceReport(deviceId));
  }
}
