/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.yswzMaterials.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.yswzMaterials.entity.StocksStockInfo;
import com.tofly.yswzMaterials.entity.dto.StockInfoQuery;
import com.tofly.yswzMaterials.service.StocksStockInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;

/**
 * 物资库存信息表
 *
 * @author JF
 * @date Tue Dec 13 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfstocksstockinfo")
@Api(tags = "物资库存信息表接口")
public class StocksStockInfoController {

  private final StocksStockInfoService stocksStockInfoService;

  /**
   * 分页查询
   *
   * @param page 分页对象
   * @param stockInfoQuery 物资库存信息表
   * @return
   */
  @GetMapping("/page")
  @ApiOperation(value = "物资库存信息，分页查询")
  public ResultRespone getStockInfoPage(Page page, @ModelAttribute StockInfoQuery stockInfoQuery) {
    return ResultRespone.success(stocksStockInfoService.getDataByPage(page, stockInfoQuery));
  }

  /**
   * 通过id查询物资库存信息表
   *
   * @param id id
   * @return ResultRespone
   */
  @GetMapping("/{id}")
  @ApiOperation(value = "通过ID查询")
  @ApiImplicitParams({
    @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
  })
  public ResultRespone getById(@PathVariable("id") Long id) {
    return ResultRespone.success(stocksStockInfoService.getById(id));
  }

  /**
   * 通过库存查询物资入库记录
   *
   * @param stockInfoId id
   * @return ResultRespone
   */
  @GetMapping("/inStorePage")
  @ApiOperation(value = "通过库存ID查询物资入库记录")
  @ApiImplicitParams({@ApiImplicitParam(name = "stockInfoId", value = "库存ID", dataType = "Long")})
  public ResultRespone getInStorePage(Page page, Long stockInfoId) {
    return ResultRespone.success(stocksStockInfoService.getInStoreDataPage(page, stockInfoId));
  }

  @GetMapping("/getReturnStorePage")
  @ApiOperation(value = "通过库存ID查询材料回收退库记录")
  @ApiImplicitParams({@ApiImplicitParam(name = "stockInfoId", value = "库存ID", dataType = "Long")})
  public ResultRespone getReturnStorePage(Page page, Long stockInfoId) {
    return ResultRespone.success(stocksStockInfoService.getReturnStoreDataPage(page, stockInfoId));
  }

  /**
   * 通过库存查询物资出库记录
   *
   * @param stockInfoId id
   * @return ResultRespone
   */
  @GetMapping("/outStorePage")
  @ApiOperation(value = "通过库存ID查询物资出库记录")
  @ApiImplicitParams({@ApiImplicitParam(name = "stockInfoId", value = "库存ID", dataType = "Long")})
  public ResultRespone getOutStorePage(Page page, Long stockInfoId) {
    return ResultRespone.success(stocksStockInfoService.getOutStoreDataPage(page, stockInfoId));
  }

  @GetMapping("/getSupplementPage")
  @ApiOperation(value = "通过库存ID查询增补出库记录")
  @ApiImplicitParams({@ApiImplicitParam(name = "stockInfoId", value = "库存ID", dataType = "Long")})
  public ResultRespone getSupplementPage(Page page, Long stockInfoId) {
    return ResultRespone.success(stocksStockInfoService.getSupplementDataPage(page, stockInfoId));
  }

  @GetMapping("/getTakeawayPage")
  @ApiOperation(value = "通过库存ID查询外卖出库记录")
  @ApiImplicitParams({@ApiImplicitParam(name = "stockInfoId", value = "库存ID", dataType = "Long")})
  public ResultRespone getTakeawayPage(Page page, Long stockInfoId) {
    return ResultRespone.success(stocksStockInfoService.getTakeawayDataPage(page, stockInfoId));
  }

  /**
   * 新增物资库存信息表
   *
   * @param stocksStockInfo 物资库存信息表
   * @return ResultRespone
   */
  @ToFlyAppLog(title = "新增物资库存信息表")
  @ApiOperation(value = "新增物资库存信息表")
  @PostMapping
  public ResultRespone save(@RequestBody StocksStockInfo stocksStockInfo) {
    return ResultRespone.success(stocksStockInfoService.save(stocksStockInfo));
  }

  /**
   * 修改物资库存信息表
   *
   * @param stocksStockInfo 物资库存信息表
   * @return ResultRespone
   */
  @ToFlyAppLog(title = "修改物资库存信息表")
  @ApiOperation(value = "修改物资库存信息表")
  @PutMapping
  public ResultRespone updateById(@RequestBody StocksStockInfo stocksStockInfo) {
    return ResultRespone.success(stocksStockInfoService.updateById(stocksStockInfo));
  }

  /**
   * 通过id删除物资库存信息表
   *
   * @param id id
   * @return ResultRespone
   */
  @ToFlyAppLog(title = "通过id删除物资库存信息表")
  @ApiOperation(value = "通过ID删除物资库存信息表")
  @ApiImplicitParams({
    @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
  })
  @DeleteMapping("/{id}")
  public ResultRespone removeById(@PathVariable Long id) {
    return ResultRespone.success(stocksStockInfoService.removeById(id));
  }

  /**
   * 通过id批量删除物资库存信息表
   *
   * @param ids id
   * @return ResultRespone
   */
  @ToFlyAppLog(title = "通过id批量删除物资库存信息表")
  @ApiOperation(value = "通过ID批量删除物资库存信息表")
  @ApiImplicitParams({
    @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
  })
  @DeleteMapping("/deleteByIds")
  public ResultRespone removeById(String ids) {
    return ResultRespone.success(stocksStockInfoService.removeByIds(Arrays.asList(ids.split(","))));
  }
}
