package com.tofly.yswzMaterials.controller;

import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.yswzMaterials.entity.StocksTakeawayInfo;
import com.tofly.yswzMaterials.entity.dto.StocksTakeawayInfoDto;
import com.tofly.yswzMaterials.entity.dto.TakeawayQueryDto;
import com.tofly.yswzMaterials.service.StocksTakeawayInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.Date;
import java.util.List;

/**
 * @author bubble
 */
@RestController
@AllArgsConstructor
@RequestMapping("/takeawayInfo")
@Api(tags = "外卖出库接口")
public class StocksTakeawayInfoController {

  private final StocksTakeawayInfoService stocksTakeawayInfoService;

  @PostMapping("/saveTakeawayInfo")
  @ApiOperation("增加外卖出库记录")
  public ResultRespone saveTakeawayInfo(StocksTakeawayInfo info, List<MultipartFile> file) {
    return ResultRespone.success(stocksTakeawayInfoService.saveTakeawayInfo(info, file));
  }

  /*@GetMapping("/list")
  @ApiOperation("查询出库记录列表")
  public ResultRespone list(StocksTakeawayInfo info){
      return ResultRespone.success(stocksTakeawayInfoService.list(Wrappers.query(info)));
  }*/

  @GetMapping("/page")
  @ApiOperation("分页查询")
  public ResultRespone page(Page page, TakeawayQueryDto infoDto) {
    String input = infoDto.getInput();
    Date startDate = infoDto.getStartDate();
    Date endDate = infoDto.getEndDate();
    Long status = infoDto.getStatus();

    Page result =
        stocksTakeawayInfoService.page(
            page,
            Wrappers.<StocksTakeawayInfo>lambdaQuery()
                .eq(status != null, StocksTakeawayInfo::getAuditStatus, status)
                .between(
                    startDate != null && endDate != null,
                    StocksTakeawayInfo::getTakeawayDate,
                    startDate,
                    endDate)
                .and(
                    StringUtils.isNotBlank(input),
                    w ->
                        w.like(StocksTakeawayInfo::getBillNo, input)
                            .or()
                            .like(StocksTakeawayInfo::getRemark, input)));
    return ResultRespone.success(result);
  }

  @DeleteMapping("/removeByIds")
  @ApiOperation("删除外卖出库记录")
  public ResultRespone removeByIds(String ids) {
    return ResultRespone.success(stocksTakeawayInfoService.removeByIds(ids));
  }

  @PutMapping("/editTakeawayStockInfo")
  @ApiOperation("修改外卖出库记录")
  public ResultRespone editTakeawayStockInfo(
      StocksTakeawayInfo info, List<MultipartFile> file) {
    return ResultRespone.success(
        stocksTakeawayInfoService.editTakeawayStockInfo(info, file));
  }
}
