package com.tofly.yswzMaterials.custom.service.impl;

import cn.hutool.core.io.IORuntimeException;
import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.yswzMaterials.config.ApiCallsBean;
import com.tofly.yswzMaterials.custom.service.CallApiService;
import com.tofly.yswzMaterials.em.Constant;
import lombok.AllArgsConstructor;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Bubble
 */
@Service
@AllArgsConstructor
public class CallApiServiceImpl implements CallApiService {

  private final ApiCallsBean apiCallsBean;

  /**
   * 获取token
   *
   * @return string
   */
  @Override
  public String getToken() {
    Map<String, Object> body = new HashMap<>(3);
    body.put("AppID", apiCallsBean.getAppId());
    body.put("Password", apiCallsBean.getPassword());

    String login = "/api/Token/P/getToken";
    String url = apiCallsBean.getIp() + login;
    String execute;
    String authorization = null;
    try {
      execute =
          HttpRequest.post(url).timeout(10000).body(JSONObject.toJSONString(body)).execute().body();
    } catch (IORuntimeException exception) {
      throw new ToflyDeniedException("请求登录接口失败：" + exception.getMessage());
    }
    if (execute != null) {
      String replace = execute.replace("\\", "");
      Map result = JSONObject.parseObject(replace.substring(1, replace.length() - 1), Map.class);
      if (CollectionUtils.isNotEmpty(result)) {
        authorization = String.valueOf(result.get("Token"));
      }
    } else {
      throw new ToflyDeniedException("请求登录接口失败：获取信息为空");
    }
    return authorization;
  }

  /**
   * 获取get请求结果
   *
   * @param url url
   * @return map
   */
  @Override
  public Map getApiResult(Object body,String url) {
    url = apiCallsBean.getIp() + url;
    try {
//      HttpRequest httpRequest;
//      //当body不为空，则使用post请求否则get请求
//      if (body != null){
//        String saveBody = url.contains("save") ? JSONObject.toJSONString(body) : String.valueOf(body);
//        //若是保存接口则转json
//        httpRequest = HttpRequest.post(url)
//                .charset("UTF-8")
//                .header("Content-Type", "application/json")
//                .header("User-Agent", "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/88.0.4324.190 Safari/537.36")
//                .header("Connection", "keep-alive")
//                .header("Accept-Language", "zh-CN,zh;q=0.9")
//                .header("Accept-Encoding", "gzip, deflate")
//                .header("Accept", "application/json")
//                .body(saveBody);
//      }else {
//        httpRequest = HttpRequest.get(url);
//      }
//      String execute = httpRequest.execute().body();
      String saveBody = url.contains("save") ? JSONObject.toJSONString(body) : "\"" + body + "\"";
      HttpClient client = HttpClients.createDefault();
      String execute = null;
      HttpResponse res;
      if (body != null){
        HttpPost post = new HttpPost(url);
        // 加入字符集设置
        StringEntity s = new StringEntity(saveBody, "UTF-8");
        s.setContentEncoding("UTF-8");
        post.addHeader("Content-Type", "application/json");
        post.addHeader("User-Agent", "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/88.0.4324.190 Safari/537.36");
        post.addHeader("Connection", "keep-alive");
        post.addHeader("Accept-Language", "zh-CN,zh;q=0.9");
        post.addHeader("Accept-Encoding", "gzip, deflate");
        post.addHeader("Accept", "application/json");
        post.setEntity(s);
        try {
          res = client.execute(post);
          execute = EntityUtils.toString(res.getEntity());
        } catch (IOException e) {
          e.printStackTrace();
        }
      }else {
        HttpGet get = new HttpGet(url);
        try {
          res = client.execute(get);
          execute = EntityUtils.toString(res.getEntity());
        } catch (IOException e) {
          e.printStackTrace();
        }
      }
      assert execute != null;
      String replace = execute.replace("\\", "");
      String substring = replace.substring(1, replace.length() - 1);
      //返回结果不为空，且返回数据中有Success代表返回成功
      if (execute.contains("Success")) {
        String success = substring.substring(substring.indexOf(":") + 1, substring.indexOf(","));
        //处理返回数据
        String message = substring.substring(substring.indexOf("\":\"")+3, substring.lastIndexOf("\""));
        if (StringUtils.isBlank(message) && Constant.YES.equals(success)){
          return null;
        }
        String jsonStr = "{\"Success\":" + success + "," + (Constant.YES.equals(success) ? "\"Message\":" + message + "}" : "\"Message\":\"" + message + "\"}");
        Map result = JSONObject.parseObject(jsonStr, Map.class);
        if (CollectionUtils.isNotEmpty(result)) {
          String successStr = String.valueOf(result.get("Success"));
          if (!Constant.YES.equals(successStr)) {
            throw new ToflyDeniedException("获取接口返回结果失败:" + result.get("Message"));
          }
        }
        return result;

      }else {
//        String errorMsg = substring.substring(substring.lastIndexOf(":") + 2, substring.indexOf("}")-1);
        throw new ToflyDeniedException("接口请求失败:" + execute);
      }
    } catch (IORuntimeException exception) {
      throw new ToflyDeniedException("请求接口失败：" + exception.getMessage());
    }
  }

  @Override
  public Map saveResult(Object body, String url) {
    url = apiCallsBean.getIp() + url;
    try {
      String execute = HttpRequest.post(url).body(String.valueOf(body)).execute().body();
      String replace = execute.replace("\\", "");
      String substring = replace.substring(1, replace.length() - 1);
      //返回结果不为空，且返回数据中有Success代表返回成功
      if (execute.contains("Success")) {
        String success = substring.substring(substring.indexOf(":") + 1, substring.indexOf(","));
        //处理返回数据
        String message = substring.substring(substring.indexOf("\":\"")+3, substring.lastIndexOf("\""));
        if (StringUtils.isBlank(message) && Constant.YES.equals(success)){
          return null;
        }
        String jsonStr = "{\"Success\":" + success + "," + (Constant.YES.equals(success) ? "\"Message\":" + message + "}" : "\"Message\":\"" + message + "\"}");
        Map result = JSONObject.parseObject(jsonStr, Map.class);
        if (CollectionUtils.isNotEmpty(result)) {
          String successStr = String.valueOf(result.get("Success"));
          if (!Constant.YES.equals(successStr)) {
            throw new ToflyDeniedException("获取接口返回结果失败:" + result.get("Message"));
          }
        }
        return result;
      }else {
        throw new ToflyDeniedException("接口请求失败:" + execute);
      }
    } catch (IORuntimeException exception) {
      throw new ToflyDeniedException("请求接口失败：" + exception.getMessage());
    }
  }

}
