package com.tofly.yswzMaterials.custom.service.impl;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.yswzMaterials.custom.mapper.StatisticsMapper;
import com.tofly.yswzMaterials.custom.service.StatisticsService;
import com.tofly.yswzMaterials.em.StocksAuditEnum;
import com.tofly.yswzMaterials.entity.*;
import com.tofly.yswzMaterials.entity.dto.StatisticsQuery;
import com.tofly.yswzMaterials.entity.vo.StatisticsTotalVo;
import com.tofly.yswzMaterials.service.*;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author bubble
 */
@Service
@AllArgsConstructor
public class StatisticsServiceImpl implements StatisticsService {

  private final StatisticsMapper statisticsMapper;
  private final StocksInstockInfoService stocksInstockInfoService;
  private final StocksOutstockInfoService stocksOutstockInfoService;
  private final StocksSupplementInfoService stocksSupplementInfoService;
  private final StocksTakeawayInfoService stocksTakeawayInfoService;
  private final StocksMaterialTypeService materialService;

  @Override
  public List<StatisticsTotalVo> recordStatistics(StatisticsQuery query) {
    return statisticsMapper.recordStatistics(query);
  }

  /**
   * 记录详情查询
   *
   * @param page page
   * @param query query
   * @return page
   */
  @Override
  public Page recordInfoPage(Page page, StatisticsQuery query) {
    Page result;

    Long queryType = query.getQueryType();
    String input = query.getInput();
    Long firm = query.getFirm();
    Long storeId = query.getStoreId();
    Long createUser = query.getCreateUser();
    Date endTime = query.getEndTime();
    Date startTime = query.getStartTime();
    Long status = query.getStatus();

    if (queryType == null) {
      throw new ToflyDeniedException("查询类型不允许为空");
    }
    if (queryType == 2) {
      result =
          stocksOutstockInfoService.page(
              page,
              Wrappers.<StocksOutstockInfo>lambdaQuery()
                  .between(
                      endTime != null && startTime != null,
                      StocksOutstockInfo::getOutstockTime,
                      startTime,
                      endTime)
                  .eq(firm != null, StocksOutstockInfo::getFirm, firm)
                  .eq(status != null, StocksOutstockInfo::getOutstockStatus, status)
                  .like(StringUtils.isNotBlank(input), StocksOutstockInfo::getOutstockNo, input)
                  .eq(createUser != null, StocksOutstockInfo::getOutstockUserId, createUser)
                  .eq(storeId != null, StocksOutstockInfo::getStoreId, storeId));
    } else if (queryType == 1 || queryType == 3) {
      int code = queryType == 1 ? 1 : 0;
      result =
          stocksSupplementInfoService.page(
              page,
              Wrappers.<StocksSupplementInfo>lambdaQuery()
                  .eq(StocksSupplementInfo::getSupplementType, code)
                  .between(
                      endTime != null && startTime != null,
                      StocksSupplementInfo::getSupplementDate,
                      startTime,
                      endTime)
                  .eq(firm != null, StocksSupplementInfo::getFirm, firm)
                  .like(
                      StringUtils.isNotBlank(input), StocksSupplementInfo::getSupplementCode, input)
                  .eq(createUser != null, StocksSupplementInfo::getSupplementUser, createUser)
                  .eq(storeId != null, StocksSupplementInfo::getStoreId, storeId));
    } else if (queryType == 4) {
      result =
          stocksTakeawayInfoService.page(
              page,
              Wrappers.<StocksTakeawayInfo>lambdaQuery()
                  .between(
                      endTime != null && startTime != null,
                      StocksTakeawayInfo::getTakeawayDate,
                      startTime,
                      endTime)
                  .eq(status != null, StocksTakeawayInfo::getAuditStatus, status)
                  .like(StringUtils.isNotBlank(input), StocksTakeawayInfo::getBillNo, input)
                  .eq(createUser != null, StocksTakeawayInfo::getTakeawayUser, createUser)
                  .eq(storeId != null, StocksTakeawayInfo::getStoreId, storeId));
    } else {
      result =
          stocksInstockInfoService.page(
              page,
              Wrappers.<StocksInstockInfo>lambdaQuery()
                  .between(
                      endTime != null && startTime != null,
                      StocksInstockInfo::getInstockTime,
                      startTime,
                      endTime)
                  .eq(firm != null, StocksInstockInfo::getFirm, firm)
                  .eq(status != null && queryType == 0, StocksInstockInfo::getInstockStatus, status)
                  .like(StringUtils.isNotBlank(input), StocksInstockInfo::getInstockNo, input)
                  .eq(
                      queryType == 0,
                      StocksInstockInfo::getInstockType,
                      StocksAuditEnum.IN_STOCK_TYPE.getCode())
                  .ne(
                      queryType == 1,
                      StocksInstockInfo::getInstockType,
                      StocksAuditEnum.IN_STOCK_TYPE.getCode())
                  .eq(createUser != null, StocksInstockInfo::getInstockUserId, createUser)
                  .eq(storeId != null, StocksInstockInfo::getStoreId, storeId));
    }
    return result;
  }

  /**
   * 物资费用汇总
   *
   * @param query 查询条件
   * @return page
   */
  @Override
  public List<Map<String, Object>> summaryOfMaterialCosts(StatisticsQuery query) {
    List<Map<String, Object>> result = new ArrayList<>();
    // 获取物资类型
    List<StocksMaterialType> materialTypeList = materialService.list();
    if (CollectionUtils.isNotEmpty(materialTypeList)) {
      Map<String, Object> data = null;
      for (StocksMaterialType stocksMaterialType : materialTypeList) {
        query.setMaterialType(stocksMaterialType.getId());
        data = new HashMap<>(8);
        data.put("materialTypeName", stocksMaterialType.getMaterialTypeName());
        data.put("materialTypeId", stocksMaterialType.getId());
        data.putAll(statisticsMapper.inStockSummary(query));
        data.putAll(statisticsMapper.outStockSummary(query));
        data.putAll(statisticsMapper.takeawayStockSummary(query));
        result.add(data);
      }
    }
    return result;
  }

  @Override
  public Page<Map<String, Object>> detailOfMaterialCosts(Page page, StatisticsQuery query) {
    Long queryType = query.getQueryType();
    if (queryType == null) {
      throw new ToflyDeniedException("查询类型不允许为空");
    }
    Page<Map<String, Object>> result = null;

    if (queryType == 0) {
      result = statisticsMapper.inStockDetail(page, query);
    } else if (queryType == 2) {
      result = statisticsMapper.outStockDetail(page, query);
    } else if (queryType == 4) {
      result = statisticsMapper.takeawayStockDetail(page, query);
    }
    return result;
  }
}
