package com.tofly.yswzMaterials.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.ftp.config.FtpUtil;
import com.tofly.yswzMaterials.em.Constant;
import com.tofly.yswzMaterials.entity.FileInfo;
import com.tofly.yswzMaterials.entity.IFile;
import com.tofly.yswzMaterials.mapper.IFileMapper;
import com.tofly.yswzMaterials.service.IFileInfoService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author JF
 */
@Service
public class IFileInfoServiceImpl extends ServiceImpl<IFileMapper, IFile>
    implements IFileInfoService {

  /** 文件上传 */
  @Override
  public List<FileInfo> saveFileList(
      List<MultipartFile> fileList, String filePathBean, String tableName) {
    if (fileList == null || fileList.size() == 0) {
      return null;
    }
    List<FileInfo> fileInfoList = new ArrayList<>();
    // 保存文件至ftp
    fileList.forEach(
        file -> {
          IFile localFile = new IFile();
          FileInfo fileInfo = new FileInfo();
          // 文件名
          String fileName = file.getOriginalFilename();
          if (StringUtils.isBlank(fileName)) {
            return;
          }
          localFile.setFileName(fileName);
          fileInfo.setFileName(fileName);
          // 文件后缀
          String suffix = fileName.substring(fileName.lastIndexOf(Constant.POINT));
          localFile.setFileFormat(suffix);
          fileInfo.setFileFormat(suffix);
          // 文件路径
          String filePath = filePathBean + IdWorker.getIdStr() + suffix;
          localFile.setFilePath(filePath);
          fileInfo.setFilePath(filePath);
          // 新文件名
          String otherName = filePath.substring(filePath.lastIndexOf("/") + 1);
          localFile.setOtherName(otherName);
          fileInfo.setOtherName(otherName);
          // 文件大小
          String fileSize =
              new BigDecimal(file.getSize())
                      .divide(BigDecimal.valueOf(1024), 1, RoundingMode.HALF_UP)
                  + "KB";
          localFile.setFileSize(fileSize);
          fileInfo.setFileSize(fileSize);
          // 关联表名称
          localFile.setTableName(tableName);
          FtpUtil.uploadFile2Ftp(file, filePath);
          this.save(localFile);
          fileInfoList.add(fileInfo);
        });
    return fileInfoList;
  }

  @Override
  public List<FileInfo> saveFileListTwo(
      List<MultipartFile> fileList, String filePathBean, String tableName) {
    if (fileList == null || fileList.size() == 0) {
      return null;
    }
    List<FileInfo> fileInfoList = new ArrayList<>();
    // 保存文件至ftp
    fileList.forEach(
        file -> {
          IFile localFile = new IFile();
          FileInfo fileInfo = new FileInfo();
          // 文件名
          String fileName = file.getOriginalFilename();
          if (StringUtils.isBlank(fileName)) {
            return;
          }
          localFile.setFileName(fileName);
          fileInfo.setFileName(fileName);
          // 文件后缀
          String suffix = fileName.substring(fileName.lastIndexOf(Constant.POINT));
          localFile.setFileFormat(suffix);
          fileInfo.setFileFormat(suffix);
          // 文件路径
          String filePath = filePathBean + IdWorker.getIdStr() + suffix;
          localFile.setFilePath(filePath);
          fileInfo.setFilePath(filePath);
          // 新文件名
          String otherName = filePath.substring(filePath.lastIndexOf("/") + 1);
          localFile.setOtherName(otherName);
          fileInfo.setOtherName(otherName);
          // 文件大小
          String fileSize =
              new BigDecimal(file.getSize())
                      .divide(BigDecimal.valueOf(1024), 1, RoundingMode.HALF_UP)
                  + "KB";
          localFile.setFileSize(fileSize);
          fileInfo.setFileSize(fileSize);
          // 关联表名称
          localFile.setTableName(tableName);
          FtpUtil.uploadFile2Ftp(file, filePath);
          this.save(localFile);
          fileInfoList.add(fileInfo);
        });
    return fileInfoList;
  }

  @Override
  public void removeFileList(List<FileInfo> fileInfoList) {
    if (fileInfoList == null || fileInfoList.size() == 0) {
      return;
    }
    List<String> filePathList =
        fileInfoList.stream()
            .map(FileInfo::getFilePath)
            .filter(Objects::nonNull)
            .collect(Collectors.toList());
    // 删除远程文件
    filePathList.forEach(FtpUtil::deleteFile);
  }

  @Override
  public String getNewAnnex(
      String nowAnnex,
      List<MultipartFile> annexList,
      String agoAnnex,
      String path,
      String tableName) {
    // 获取传入的文件集合
    List<FileInfo> nowAnnexFileInfos = new ArrayList<>();
    if (StringUtils.isNotBlank(nowAnnex)) {
      nowAnnexFileInfos = JSON.parseArray(nowAnnex, FileInfo.class);
    }
    // 获取之前的文件集合
    List<FileInfo> agoAnnexFileInfos = new ArrayList<>();
    if (StringUtils.isNotBlank(agoAnnex)) {
      agoAnnexFileInfos = JSON.parseArray(agoAnnex, FileInfo.class);
    }
    // 保存文件并获得其文件集合
    List<FileInfo> newAnnexFileInfos = new ArrayList<>();
    if (annexList != null && annexList.size() != 0) {
      newAnnexFileInfos = this.saveFileList(annexList, path, tableName);
    }
    // 获取需要删除的文件集合并删除
    List<FileInfo> finalNowAnnexFileInfos = nowAnnexFileInfos;
    List<FileInfo> reduce =
        agoAnnexFileInfos.stream()
            .filter(item -> !finalNowAnnexFileInfos.contains(item))
            .collect(Collectors.toList());
    this.removeFileList(reduce);
    // 获取最新的文件集合
    newAnnexFileInfos.addAll(nowAnnexFileInfos);
    if (newAnnexFileInfos.size() != 0) {
      return JSON.toJSONString(newAnnexFileInfos);
    }
    return null;
  }

  /**
   * 文件上传更新操作方法，支持删除旧文件，添加新文件
   *
   * @param fileList 新上传的文件列表
   * @param newAnnex 修改编辑过的附件集合json字符串
   * @param oldAnnex 数据库中的附件集合json字符串
   * @param folder 上传路径文件夹
   * @param tableName 上传附件对应的数据库表名
   * @return 上传文件更新删除之后的新附件集合json字符串
   */
  @Override
  public String uploadFiles(
      List<MultipartFile> fileList,
      String newAnnex,
      String oldAnnex,
      String folder,
      String tableName) {
    String annex = null;
    // 编辑的时候需要删除文件
    if (org.springframework.util.StringUtils.hasText(newAnnex)
        && org.springframework.util.StringUtils.hasText(oldAnnex)
        && !newAnnex.equals(oldAnnex)) {
      List<FileInfo> fileInfos = JSONObject.parseArray(newAnnex, FileInfo.class);
      List<FileInfo> fileInfosDb = JSONObject.parseArray(oldAnnex, FileInfo.class);
      // 获取差集删除部分
      List<FileInfo> collect =
          fileInfosDb.stream()
              .filter(fileInfo -> !fileInfos.contains(fileInfo))
              .collect(Collectors.toList());
      // 如果差集存在
      if (!CollectionUtils.isEmpty(collect)) {
        // 删除文件
        removeFileList(collect);
      }
      // 获取交集部分
      List<FileInfo> collect1 =
          fileInfosDb.stream().filter(fileInfos::contains).collect(Collectors.toList());
      if (!CollectionUtils.isEmpty(collect1)) {
        annex = JSONObject.toJSONString(collect1);
      }
    } else if (!org.springframework.util.StringUtils.isEmpty(newAnnex)
        && newAnnex.equals(oldAnnex)) {
      annex = oldAnnex;
    }
    // 上传新文件
    if (fileList != null && fileList.size() != 0 && !CollectionUtils.isEmpty(fileList)) {
      List<FileInfo> fileInfos = saveFileList(fileList, folder, tableName);
      // 以前没有上传过文件
      if (!CollectionUtils.isEmpty(fileInfos)
          && !org.springframework.util.StringUtils.hasText(annex)) {
        // 直接返回现在新文件对象json
        return JSONObject.toJSONString(fileInfos);
        // 以前上传过文件，则在以前的数据上进行累加
      } else if (!CollectionUtils.isEmpty(fileInfos)
          && org.springframework.util.StringUtils.hasText(annex)) {
        List<FileInfo> list = JSONObject.parseArray(annex, FileInfo.class);
        list.addAll(fileInfos);
        return JSONObject.toJSONString(list);
      }
    }
    if (annex == null && newAnnex == "[]") {
      annex = "";
    }
    return annex;
  }
}
