package com.tofly.yswzMaterials.service.impl;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.yswzMaterials.custom.service.CallApiService;
import com.tofly.yswzMaterials.custom.service.CommonUtilService;
import com.tofly.yswzMaterials.em.JinDieUrlEnum;
import com.tofly.yswzMaterials.em.StocksAuditEnum;
import com.tofly.yswzMaterials.entity.*;
import com.tofly.yswzMaterials.entity.dto.CommonQueryDto;
import com.tofly.yswzMaterials.entity.vo.*;
import com.tofly.yswzMaterials.mapper.LossMapper;
import com.tofly.yswzMaterials.service.IFileInfoService;
import com.tofly.yswzMaterials.service.LossDetailService;
import com.tofly.yswzMaterials.service.LossService;
import com.tofly.yswzMaterials.service.StocksAuditService;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author Bubble
 */
@Service
public class LossServiceImpl extends ServiceImpl<LossMapper, Loss> implements LossService {
    @Autowired
    private CallApiService callApiService;
    @Autowired
    private LossDetailService lossDetailService;
    @Autowired
    private IFileInfoService fileInfoService;
    @Autowired
    private CommonUtilService commonUtilService;
/*    @Autowired
    private StocksAuditService stocksAuditService;*/

    @Value("${yswz.ftp.loss}")
    private String path;

    /**
     * 添加盘亏单
     *
     * @param loss 盘亏
     * @return boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean insertLoss(Loss loss,List<MultipartFile> file) {
        loss.setBillNo(commonUtilService.getBillNo(6,null));
        int count = count(Wrappers.<Loss>lambdaQuery().eq(Loss::getBillNo, loss.getBillNo()));
        if (count > 0){
            throw new ToflyDeniedException("登记单号重复，清重新编辑");
        }
        //登记时为未审核
        loss.setAuditStatus(StocksAuditEnum.NOT_REVIEWED.getCode());
        List<LossDetail> detailList = loss.getDetailList();
        if (CollectionUtils.isNotEmpty(file)){
            loss.setAnnex(fileInfoService.uploadFiles(file, loss.getAnnex(), loss.getAnnex(), path, "TF_JINDIE_LOSS"));
        }
        //添加入库单
        save(loss);
        //添加入库单详情
        detailList.forEach(detail->{
            detail.setLossId(loss.getId());
            lossDetailService.save(detail);
        });
        return true;
    }

    /**
     * 修改盘亏单
     *
     * @param loss loss
     * @param file file
     * @return boolean
     */
    @Override
    public boolean updateLoss(Loss loss, List<MultipartFile> file) {
        Loss byId = getById(loss.getId());
        Long auditStatus = loss.getAuditStatus();
        if (!auditStatus.equals(StocksAuditEnum.NOT_REVIEWED.getCode())
                && !auditStatus.equals(StocksAuditEnum.HSIANG_HSIEN.getCode())) {
            throw new ToflyDeniedException("非未审核与驳回无法修改");
        }
        if (CollectionUtils.isNotEmpty(file)){
            loss.setAnnex(
                    fileInfoService.uploadFiles(
                            file, loss.getAnnex(),byId.getAnnex(), path, "TF_JINDIE_LOSS"));
        }
        updateById(loss);
        lossDetailService.updateBatchById(loss.getDetailList());
        return true;
    }

    /**
     * 删除盘亏单
     *
     * @param ids ids
     * @return boolean
     */
    @Override
    public boolean deleteLoss(String ids) {

        List<String> idList = Arrays.asList(ids.split(","));
        List<Loss> lossList = list(Wrappers.<Loss>lambdaQuery().in(Loss::getId, idList));
        if (CollectionUtils.isNotEmpty(lossList)){
            long count = lossList.stream().filter(s -> !s.getAuditStatus().equals(StocksAuditEnum.NOT_REVIEWED.getCode())
                    && !s.getAuditStatus().equals(StocksAuditEnum.HSIANG_HSIEN.getCode())).count();
            if (count > 0){
                throw new ToflyDeniedException("非未审核或驳回入库单，无法删除");
            }
        }
        //清除审核记录
/*        stocksAuditService.remove(
                Wrappers.<StocksAudit>lambdaQuery()
                        .eq(StocksAudit::getAuditType, StocksAuditEnum.LOSS_CONFIG_TYPE.getCode())
                        .in(StocksAudit::getTableId, idList));
        removeByIds(idList);*/
        lossDetailService.remove(Wrappers.<LossDetail>lambdaQuery()
                .in(LossDetail::getLossId, idList));
        return true;
    }



    /**
     * 分页查询盘亏单
     *
     * @param page page
     * @param dto  dto
     * @return page
     */
    @Override
    public Page<Loss> getPageInfo(Page page, CommonQueryDto dto) {
        Date startTime = dto.getStartTime();
        Date endTime = dto.getEndTime();
        String input = dto.getInput();
        return page(page,Wrappers.<Loss>lambdaQuery()
                .between(startTime != null && endTime != null,Loss::getFDate, startTime, endTime)
                .like(StringUtils.isNotBlank(input),Loss::getBillNo, input)
                .orderByDesc(Loss::getFDate));
    }

    /**
     * 查询盘亏单
     *
     * @param code code
     * @return LossParamVo
     */
    @Override
    public LossParamVo getLossInfo(String code) {
        String tokenInfo = callApiService.getToken();
        String url = JinDieUrlEnum.GET_LOSS_URL.getName() + "?token=" + tokenInfo + "&json=" + code;
        Map apiResult = callApiService.getApiResult(null, url);
        if (CollectionUtils.isNotEmpty(apiResult)){
            return JSONObject.parseObject(String.valueOf(apiResult.get("Message")), LossParamVo.class);
        }
        return null;
    }


}
