package com.tofly.yswzMaterials.service.impl;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.yswzMaterials.custom.service.CallApiService;
import com.tofly.yswzMaterials.custom.service.CommonUtilService;
import com.tofly.yswzMaterials.em.JinDieUrlEnum;
import com.tofly.yswzMaterials.em.StocksAuditEnum;
import com.tofly.yswzMaterials.entity.Loss;
import com.tofly.yswzMaterials.entity.Picking;
import com.tofly.yswzMaterials.entity.PickingDetail;
import com.tofly.yswzMaterials.entity.StocksAudit;
import com.tofly.yswzMaterials.entity.dto.CommonQueryDto;
import com.tofly.yswzMaterials.entity.vo.PickingParamVo;
import com.tofly.yswzMaterials.mapper.PickingMapper;
import com.tofly.yswzMaterials.service.IFileInfoService;
import com.tofly.yswzMaterials.service.PickingDetailService;
import com.tofly.yswzMaterials.service.PickingService;
import com.tofly.yswzMaterials.service.StocksAuditService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author Bubble
 */
@Service
public class PickingServiceImpl extends ServiceImpl<PickingMapper, Picking> implements PickingService {

    @Autowired private PickingDetailService pickingDetailService;
    @Autowired private CallApiService callApiService;
    @Autowired private IFileInfoService fileInfoService;
    @Autowired private CommonUtilService commonUtilService;
//    @Autowired private StocksAuditService stocksAuditService;

    @Value("${yswz.ftp.stocksOutstockInfo}")
    private String path;

    /**
     * 新增外购入库单
     *
     * @param picking picking
     * @return boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean insertPicking(Picking picking, List<MultipartFile> file) {
        Long type = picking.getPickingType();
        picking.setBillNo(commonUtilService.getBillNo(type == 0 ? 2 : 4, Math.toIntExact(type)));
        List<PickingDetail> detailList = picking.getDetailList();
        picking.setAuditStatus(StocksAuditEnum.NOT_REVIEWED.getCode());
        if (CollectionUtils.isNotEmpty(file)){
            picking.setAnnex(
                    fileInfoService.uploadFiles(
                            file, picking.getAnnex(), picking.getAnnex(), path, "TF_JINDIE_PICKING"));
        }
        //添加入库单
        save(picking);
        //添加入库单详情
        detailList.forEach(detail->{
            detail.setPickingId(picking.getId());
            pickingDetailService.save(detail);
        });
        return true;
    }

    /**
     * 修改出库领料单
     *
     * @param picking picking
     * @param file    file
     * @return boolean
     */
    @Override
    public boolean updatePicking(Picking picking, List<MultipartFile> file) {
        Picking byId = getById(picking.getId());
        Long auditStatus = picking.getAuditStatus();
        if (!auditStatus.equals(StocksAuditEnum.NOT_REVIEWED.getCode())
                && !auditStatus.equals(StocksAuditEnum.HSIANG_HSIEN.getCode())) {
            throw new ToflyDeniedException("非未审核与驳回无法修改");
        }
        if (CollectionUtils.isNotEmpty(file)){
            picking.setAnnex(
                    fileInfoService.uploadFiles(
                            file, picking.getAnnex(),byId.getAnnex(), path, "TF_JINDIE_PICKING"));
        }

        updateById(picking);
        pickingDetailService.updateBatchById(picking.getDetailList());
        return true;
    }

    /**
     * 获取出库领料信息
     *
     * @param code 编码
     * @return PickingParamVo
     */
    @Override
    public PickingParamVo getPickingInfo(String code) {
        String tokenInfo = callApiService.getToken();
        String url = JinDieUrlEnum.GET_PICKING_URL.getName() + "?token=" + tokenInfo + "&json=" + code;
        Map apiResult = callApiService.getApiResult(null, url);
        if (CollectionUtils.isNotEmpty(apiResult)){
            return JSONObject.parseObject(String.valueOf(apiResult.get("Message")), PickingParamVo.class);
        }
        return null;
    }

    /**
     * 分页查询出库领料单
     *
     * @param page page
     * @param dto  dto
     * @return page
     */
    @Override
    public Page<Picking> getPageInfo(Page page, CommonQueryDto dto) {
        Long auditStatus = dto.getAuditStatus();
        Date startTime = dto.getStartTime();
        Date endTime = dto.getEndTime();
        String input = dto.getInput();
        Long type = dto.getType();
        return page(page, Wrappers.<Picking>lambdaQuery()
                .like(StringUtils.isNotBlank(input),Picking::getBillNo, input)
                .eq(type != null, Picking::getPickingType, type)
                .eq(auditStatus != null, Picking::getAuditStatus, auditStatus)
                .between(startTime != null && endTime != null,Picking::getFDate, startTime, endTime));
    }

    /**
     * 删除出库领料单
     *
     * @param ids ids
     * @return boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean delete(String ids) {
        List<String> idList = Arrays.asList(ids.split(","));
        List<Picking> pickingList = list(Wrappers.<Picking>lambdaQuery().in(Picking::getId, idList));
        if (CollectionUtils.isNotEmpty(pickingList)){
            long count = pickingList.stream().filter(s -> !s.getAuditStatus().equals(StocksAuditEnum.NOT_REVIEWED.getCode())
                    && !s.getAuditStatus().equals(StocksAuditEnum.HSIANG_HSIEN.getCode())).count();
            if (count > 0){
                throw new ToflyDeniedException("非未审核或驳回入库单，无法删除");
            }
        }
        //清除审核记录
/*        stocksAuditService.remove(
                Wrappers.<StocksAudit>lambdaQuery()
                        .eq(StocksAudit::getAuditType, StocksAuditEnum.OUT_STOCK_CONFIG_TYPE.getCode())
                        .in(StocksAudit::getTableId, idList));*/
        removeByIds(idList);
        pickingDetailService.remove(Wrappers.<PickingDetail>lambdaQuery()
                .in(PickingDetail::getPickingId, idList));
        return true;
    }
}
