package com.tofly.yswzMaterials.service.impl;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.yswzMaterials.custom.service.CallApiService;
import com.tofly.yswzMaterials.custom.service.CommonUtilService;
import com.tofly.yswzMaterials.em.JinDieUrlEnum;
import com.tofly.yswzMaterials.em.StocksAuditEnum;
import com.tofly.yswzMaterials.entity.*;
import com.tofly.yswzMaterials.entity.dto.CommonQueryDto;
import com.tofly.yswzMaterials.entity.vo.ProfitParamVo;
import com.tofly.yswzMaterials.mapper.ProfitMapper;
import com.tofly.yswzMaterials.service.IFileInfoService;
import com.tofly.yswzMaterials.service.ProfitDetailService;
import com.tofly.yswzMaterials.service.ProfitService;
import com.tofly.yswzMaterials.service.StocksAuditService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.util.*;

/**
 * @author Bubble
 */
@Service
public class ProfitServiceImpl extends ServiceImpl<ProfitMapper, Profit> implements ProfitService {
    @Autowired
    private CallApiService callApiService;
    @Autowired
    private ProfitDetailService profitDetailService;
    @Autowired
    private IFileInfoService fileInfoService;
    @Autowired
    private CommonUtilService commonUtilService;

/*    @Autowired
    private StocksAuditService stocksAuditService;*/

    @Value("${yswz.ftp.profit}")
    private String path;
    /**
     * 插入盘盈单
     *
     * @param profit 盘盈
     * @return boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean insertProfit(Profit profit,List<MultipartFile> file) {
        profit.setBillNo(commonUtilService.getBillNo(5,null));
        //登记时为未审核
        profit.setAuditStatus(StocksAuditEnum.NOT_REVIEWED.getCode());
        List<ProfitDetail> detailList = profit.getDetailList();
        if (CollectionUtils.isNotEmpty(file)){
            profit.setAnnex(fileInfoService.uploadFiles(file, profit.getAnnex(), profit.getAnnex(), path, "TF_JINDIE_LOSS"));
        }
        //添加入库单
        save(profit);
        //添加入库单详情
        detailList.forEach(detail->{
            detail.setProfitId(profit.getId());
            profitDetailService.save(detail);
        });
        return true;
    }

    /**
     * 更新盘盈单
     *
     * @param profit profit
     * @param file   file
     * @return boolean
     */
    @Override
    public boolean updateProfit(Profit profit, List<MultipartFile> file) {
        Profit byId = getById(profit.getId());
        Long auditStatus = profit.getAuditStatus();
        if (!auditStatus.equals(StocksAuditEnum.NOT_REVIEWED.getCode())
                && !auditStatus.equals(StocksAuditEnum.HSIANG_HSIEN.getCode())) {
            throw new ToflyDeniedException("非未审核与驳回无法修改");
        }
        if (CollectionUtils.isNotEmpty(file)){
            profit.setAnnex(
                    fileInfoService.uploadFiles(
                            file, profit.getAnnex(),byId.getAnnex(), path, "TF_JINDIE_LOSS"));
        }
        updateById(profit);
        return profitDetailService.updateBatchById(profit.getDetailList());
    }

    /**
     * 删除盘盈单
     *
     * @param ids ids
     * @return boolean
     */
    @Override
    public boolean deleteProfit(String ids) {

        List<String> idList = Arrays.asList(ids.split(","));
        List<Profit> profitList =  list(Wrappers.<Profit>lambdaQuery().in(Profit::getId, idList));
        if (CollectionUtils.isNotEmpty(profitList)){
            long count = profitList.stream().filter(s -> !s.getAuditStatus().equals(StocksAuditEnum.NOT_REVIEWED.getCode())
                    && !s.getAuditStatus().equals(StocksAuditEnum.HSIANG_HSIEN.getCode())).count();
            if (count > 0){
                throw new ToflyDeniedException("非未审核或驳回入库单，无法删除");
            }
        }
        //清除审核记录
/*        stocksAuditService.remove(
                Wrappers.<StocksAudit>lambdaQuery()
                        .eq(StocksAudit::getAuditType, StocksAuditEnum.PROFIT_CONFIG_TYPE.getCode())
                        .in(StocksAudit::getTableId, idList));*/
        removeByIds(idList);
        profitDetailService.remove(Wrappers.<ProfitDetail>lambdaQuery()
                .in(ProfitDetail::getProfitId, idList));
        return true;
    }

    /**
     * 根据id查询盘盈单
     *
     * @param id id
     * @return profit
     */
    @Override
    public Profit getProfitById(Long id) {
        return getById(id);
    }

    /**
     * 分页查询
     *
     * @param page page
     * @param dto  dto
     * @return page
     */
    @Override
    public Page<Profit> getPageInfo(Page page, CommonQueryDto dto) {
        Date startTime = dto.getStartTime();
        Date endTime = dto.getEndTime();
        String input = dto.getInput();
        return page(page,Wrappers.<Profit>lambdaQuery()
                .between(startTime != null && endTime != null,Profit::getFDate, startTime, endTime)
                .like(StringUtils.isNotBlank(input),Profit::getBillNo, input)
                .orderByDesc(Profit::getFDate));
    }

    /**
     * 查询盘盈单信息
     *
     * @param code code
     * @return ProfitParamVo
     */
    @Override
    public ProfitParamVo getProfitInfo(String code) {
        String tokenInfo = callApiService.getToken();
        String url = JinDieUrlEnum.GET_PROFIT_URL.getName() + "?token=" + tokenInfo + "&json=" + code;
        Map apiResult = callApiService.getApiResult(null, url);
        if (CollectionUtils.isNotEmpty(apiResult)){
            return JSONObject.parseObject(String.valueOf(apiResult.get("Message")), ProfitParamVo.class);
        }
        return null;
    }
}
