package com.tofly.yswzMaterials.service.impl;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.yswzMaterials.entity.Loss;
import com.tofly.yswzMaterials.entity.Picking;
import com.tofly.yswzMaterials.entity.Profit;
import com.tofly.yswzMaterials.entity.Storage;
import com.tofly.yswzMaterials.entity.dto.ReportQueryDto;
import com.tofly.yswzMaterials.mapper.ReportMapper;
import com.tofly.yswzMaterials.service.*;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Bubble
 */
@Service
@AllArgsConstructor
public class ReportServiceImpl implements ReportService {

  private final ReportMapper reportMapper;
  private final StorageService storageService;
  private final PickingService pickingService;
  private final ProfitService profitService;
  private final LossService lossService;

  /**
   * 统计报表-出入库单统计
   *
   * @param query query
   * @return list
   */
  @Override
  public List<Map<String, Object>> ticketsTotalCount(ReportQueryDto query) {
    return reportMapper.ticketsTotalCount(query);
  }

  /**
   * 材料数量成本统计
   *
   * @return map
   */
  @Override
  public Map<String, Object> getMaterialsCount(ReportQueryDto query) {
    Map<String, Object> result = new HashMap<>(6);
    result.put("storage", reportMapper.storageMaterialsCount(query, 0));
    result.put("picking", reportMapper.pickingMaterialsCount(query, 0));
    result.put("return", reportMapper.storageMaterialsCount(query, 1));
    result.put("takeaway", reportMapper.pickingMaterialsCount(query, 1));
    result.put("loss", reportMapper.lossMaterialsCount(query));
    result.put("profit", reportMapper.profitMaterialsCount(query));
    return result;
  }

  @Override
  public Page queryMaterialsDetail(Page page,ReportQueryDto query) {
    Page result = new Page();
    Integer type = query.getType();
    List<Integer> idsList = reportMapper.queryTicketsId(query);
    if (CollectionUtils.isNotEmpty(idsList)){
      if (type == 0 || type == 2){
        result = storageService.page(page, Wrappers.<Storage>lambdaQuery()
                .eq(type == 0, Storage::getStorageType, 0)
                .eq(type == 2, Storage::getStorageType, 1)
                .like(StringUtils.isNotBlank(query.getInput()),Storage::getBillNo, query.getInput())
                .in(Storage::getId, idsList));
      }else if (type ==1 || type == 3){
        result = pickingService.page(page, Wrappers.<Picking>lambdaQuery()
                .eq(type == 1, Picking::getPickingType, 0)
                .eq(type == 1, Picking::getPickingType, 1)
                .like(StringUtils.isNotBlank(query.getInput()),Picking::getBillNo, query.getInput())
                .in(Picking::getId, idsList));
      }else if (type == 4){
        result = profitService.page(page, Wrappers.<Profit>lambdaQuery()
                .like(StringUtils.isNotBlank(query.getInput()),Profit::getBillNo, query.getInput())
                .in(Profit::getId, idsList));
      }else if (type == 5){
        result = lossService.page(page, Wrappers.<Loss>lambdaQuery()
                .like(StringUtils.isNotBlank(query.getInput()),Loss::getBillNo, query.getInput())
                .in(Loss::getId, idsList));
      }
    }
    return result;
  }
}
