package com.tofly.yswzMaterials.service.impl;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.yswzMaterials.em.Constant;
import com.tofly.yswzMaterials.entity.StocksAuditConfig;
import com.tofly.yswzMaterials.mapper.StocksAuditConfigMapper;
import com.tofly.yswzMaterials.service.StocksAuditConfigService;
import org.apache.poi.util.StringUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author bubble
 */
@Service
public class StocksAuditConfigServiceImpl
    extends ServiceImpl<StocksAuditConfigMapper, StocksAuditConfig>
    implements StocksAuditConfigService {
  /**
   * 添加审批流程
   *
   * @param config 配置
   * @return true
   */
  @Override
  @Transactional(rollbackFor = Exception.class)
  public boolean insertAuditConfig(StocksAuditConfig config) {
    if (config == null) {
      throw new ToflyDeniedException("配置信息为空");
    }

    Long type = config.getType();
    Long auditLevel = config.getAuditLevel();
    Integer num = baseMapper.countAuditNumByType(type);
    if (num > 0) {
      throw new ToflyDeniedException("当前配置类型存在" + num + "条未审核数据,无法添加");
    }

    Integer typeMaxLevel = baseMapper.getTypeMaxLevel(type);
    if (typeMaxLevel == null && !auditLevel.equals(Constant.ONE)) {
      throw new ToflyDeniedException("等级输入非法");
    } else if (typeMaxLevel != null && typeMaxLevel + 1 != auditLevel) {
      throw new ToflyDeniedException("等级输入非法");
    }
    StocksAuditConfig auditConfig =
        getOne(
            Wrappers.<StocksAuditConfig>lambdaQuery()
                .eq(StocksAuditConfig::getType, type)
                .eq(StocksAuditConfig::getAuditLevel, auditLevel));

    if (auditConfig != null) {
      throw new ToflyDeniedException("该等级审批流程已存在");
    }
    return save(config);
  }

  /**
   * 删除审批配置
   *
   * @param ids ids
   * @return boolean
   */
  @Override
  @Transactional(rollbackFor = Exception.class)
  public boolean deletesAuditConfig(String ids) {

    List<String> idList = Arrays.asList(ids.split(","));
    // 查询审核信息
    List<StocksAuditConfig> configList = listByIds(idList);

    for (StocksAuditConfig auditConfig : configList) {
      StocksAuditConfig config =
          getOne(
              Wrappers.<StocksAuditConfig>lambdaQuery()
                  .select(StocksAuditConfig::getId)
                  .eq(StocksAuditConfig::getAuditLevel, auditConfig.getAuditLevel() + 1)
                  .eq(StocksAuditConfig::getType, auditConfig.getType()));
      if (config != null) {
        throw new ToflyDeniedException("当前配置存在高等级配置未删除");
      }
    }

    if (CollectionUtils.isNotEmpty(configList)) {
      Map<Long, List<StocksAuditConfig>> typeGroup =
          configList.stream().collect(Collectors.groupingBy(StocksAuditConfig::getType));
      List<String> typeList = new ArrayList<>();
      typeGroup
          .keySet()
          .forEach(
              t -> {
                if (baseMapper.countAuditNumByType(t) > 0) {
                  typeList.add(baseMapper.queryTypeName(t));
                }
              });
      if (CollectionUtils.isNotEmpty(typeList)) {
        throw new ToflyDeniedException("模块：" + StringUtil.join(",", typeList) + "存在未审核数据,无法删除");
      }
    }

    return removeByIds(idList);
  }

  /**
   * 修改配置信息
   *
   * @param config config
   * @return boolean
   */
  @Override
  public boolean updateAuditConfig(StocksAuditConfig config) {
    if (config.getType() == null) {
      throw new ToflyDeniedException("未选择审批类型");
    }
    Integer integer = baseMapper.countAuditNumByType(config.getType());
    if (integer > 0) {
      throw new ToflyDeniedException("当前配置类型存在" + integer + "条未审核数据,无法修改");
    }
    return updateById(config);
  }

  /**
   * 获取审核信息列表
   *
   * @return list
   */
  @Override
  public List<StocksAuditConfig> getAuditConfigList() {
    return list();
  }
}
