package com.tofly.yswzMaterials.service.impl;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.yswzMaterials.custom.service.CallApiService;
import com.tofly.yswzMaterials.em.Constant;
import com.tofly.yswzMaterials.em.JinDieUrlEnum;
import com.tofly.yswzMaterials.em.StocksAuditEnum;
import com.tofly.yswzMaterials.entity.*;
import com.tofly.yswzMaterials.entity.vo.*;
import com.tofly.yswzMaterials.mapper.StocksAuditMapper;
import com.tofly.yswzMaterials.service.*;
import lombok.AllArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.util.*;

/**
 * @author bubble
 */
@Service
@AllArgsConstructor
public class StocksAuditServiceImpl extends ServiceImpl<StocksAuditMapper, StocksAudit>
    implements StocksAuditService {

  private final StocksAuditConfigService stocksAuditConfigService;
  private final StocksTakeawayInfoService stocksTakeawayInfoService;
  private final StocksTakeawayDetailService stocksTakeawayDetailService;
  private final StocksStockInfoService stockInfoService;
  private final StorageService storageService;
  private final StorageDetailService storageDetailService;
  private final CallApiService callApiService;
  private final PickingService pickingService;
  private final ProfitService profitService;
  private final LossService lossService;

  /*  private static final String MESSAGE_PUSH = "TF_YWPN_MESSAGEPUSH_W";
  private static final String MESSAGE_RECIPIENT = "TF_YWPN_MESSAGERECIPIENT_W";*/

  /**
   * 发起审核
   *
   * @param type 审核类型：出库、入库等，取字典表
   * @param id 审核表id
   * @return Boolean
   */
  @Override
  @Transactional(rollbackFor = Exception.class)
  public boolean initiateReview(Long type, Long id) {
    if (type == null) {
      throw new ToflyDeniedException("未选择配置类型");
    }
    if (id == null) {
      throw new ToflyDeniedException("未选择审核上报对象");
    }

    // 获取对应类型的审核配置的一级审核人
    StocksAuditConfig auditConfig =
        stocksAuditConfigService.getOne(
            Wrappers.<StocksAuditConfig>lambdaQuery()
                .eq(StocksAuditConfig::getType, type)
                .eq(StocksAuditConfig::getAuditLevel, Constant.ONE)
                .orderByAsc(StocksAuditConfig::getAuditLevel));

    if (auditConfig == null) {
      throw new ToflyDeniedException("请确定该模块是否已配置审批流程");
    }
    if (type.equals(StocksAuditEnum.IN_STOCK_CONFIG_TYPE.getCode()) || type.equals(StocksAuditEnum.RETURN_STOCK_CONFIG_TYPE.getCode())) {
      Storage storage = storageService.getById(id);
      if (storage == null) {
        throw new ToflyDeniedException("未找到该记录");
      }
      Long auditStatus = storage.getAuditStatus();
      if (!auditStatus.equals(StocksAuditEnum.NOT_REVIEWED.getCode())
          && !auditStatus.equals(StocksAuditEnum.HSIANG_HSIEN.getCode())) {
        throw new ToflyDeniedException("该记录已审核完成或正常审核，无法发起审批");
      }
      storage.setAuditStatus(StocksAuditEnum.UNDER_REVIEW.getCode());
      storage.setAuditUser(auditConfig.getAuditUser());
      storage.setAuditDept(auditConfig.getAuditDept());
      // 修改出库表的当前审核人
      storageService.updateById(storage);

    } else if (type.equals(StocksAuditEnum.OUT_STOCK_CONFIG_TYPE.getCode()) || type.equals(StocksAuditEnum.ADD_STOCK_CONFIG_TYPE.getCode())) {
      Picking picking = pickingService.getById(id);
      if (picking == null) {
        throw new ToflyDeniedException("未找到该记录");
      }
      Long status = picking.getAuditStatus();
      if (!status.equals(StocksAuditEnum.NOT_REVIEWED.getCode())
          && !status.equals(StocksAuditEnum.HSIANG_HSIEN.getCode())) {
        throw new ToflyDeniedException("该记录已审核完成或正常审核，无法发起审批");
      }
      // 修改出库表的当前审核人
      pickingService.update(
          Wrappers.<Picking>lambdaUpdate()
              .eq(Picking::getId, picking.getId())
              .set(Picking::getAuditStatus, StocksAuditEnum.UNDER_REVIEW.getCode())
              .set(Picking::getAuditUser, auditConfig.getAuditUser())
              .set(Picking::getAuditDept, auditConfig.getAuditDept()));
    } else if (type.equals(StocksAuditEnum.TAKEAWAY_OUT_STOCK_TYPE.getCode())){
      StocksTakeawayInfo takeawayInfo = stocksTakeawayInfoService.getById(id);
      if (takeawayInfo == null) {
        throw new ToflyDeniedException("未找到该记录");
      }
      Long takeawayStatus = takeawayInfo.getAuditStatus();
      if (!takeawayStatus.equals(StocksAuditEnum.NOT_REVIEWED.getCode())
          && !takeawayStatus.equals(StocksAuditEnum.HSIANG_HSIEN.getCode())) {
        throw new ToflyDeniedException("该记录已审核完成或正常审核，无法发起审批");
      }
      // 修改出库表的当前审核人
      stocksTakeawayInfoService.update(
          Wrappers.<StocksTakeawayInfo>lambdaUpdate()
              .eq(StocksTakeawayInfo::getId, takeawayInfo.getId())
              .set(StocksTakeawayInfo::getAuditStatus, StocksAuditEnum.UNDER_REVIEW.getCode())
              .set(StocksTakeawayInfo::getAuditUser, auditConfig.getAuditUser())
              .set(StocksTakeawayInfo::getAuditDept, auditConfig.getAuditDept()));
    }else if (type.equals(StocksAuditEnum.PROFIT_CONFIG_TYPE.getCode())){
      Profit profit = profitService.getById(id);
      if (Objects.isNull(profit)) {
        throw new ToflyDeniedException("未找到该记录");
      }
      Long profitStatus = profit.getAuditStatus();
      if (!profitStatus.equals(StocksAuditEnum.NOT_REVIEWED.getCode())
              && !profitStatus.equals(StocksAuditEnum.HSIANG_HSIEN.getCode())) {
        throw new ToflyDeniedException("该记录已审核完成或正常审核，无法发起审批");
      }
      // 修改出库表的当前审核人
      profitService.update(
              Wrappers.<Profit>lambdaUpdate()
                      .eq(Profit::getId, profit.getId())
                      .set(Profit::getAuditStatus, StocksAuditEnum.UNDER_REVIEW.getCode())
                      .set(Profit::getAuditUser, auditConfig.getAuditUser())
                      .set(Profit::getAuditDept, auditConfig.getAuditDept()));
    }else if (type.equals(StocksAuditEnum.LOSS_CONFIG_TYPE.getCode())){
      Loss loss = lossService.getById(id);
      if (Objects.isNull(loss)) {
        throw new ToflyDeniedException("未找到该记录");
      }
      Long lossStatus = loss.getAuditStatus();
      if (!lossStatus.equals(StocksAuditEnum.NOT_REVIEWED.getCode())
              && !lossStatus.equals(StocksAuditEnum.HSIANG_HSIEN.getCode())) {
        throw new ToflyDeniedException("该记录已审核完成或正常审核，无法发起审批");
      }
      // 修改出库表的当前审核人
      lossService.update(
              Wrappers.<Loss>lambdaUpdate()
                      .eq(Loss::getId, loss.getId())
                      .set(Loss::getAuditStatus, StocksAuditEnum.UNDER_REVIEW.getCode())
                      .set(Loss::getAuditUser, auditConfig.getAuditUser())
                      .set(Loss::getAuditDept, auditConfig.getAuditDept()));
    }
    List<StocksAudit> auditList = list(Wrappers.<StocksAudit>lambdaQuery()
                .eq(StocksAudit::getAuditType, type)
                .eq(StocksAudit::getTableId, id)
                .select(StocksAudit::getSort)
                .orderByDesc(StocksAudit::getId));
    // 赋值
    StocksAudit stocksAudit = new StocksAudit();
    stocksAudit.setAuditLevel(auditConfig.getAuditLevel());
    stocksAudit.setAuditDept(auditConfig.getAuditDept());
    stocksAudit.setAuditUser(auditConfig.getAuditUser());
    stocksAudit.setAuditType(auditConfig.getType());
    stocksAudit.setTableId(id);
    stocksAudit.setSort(CollectionUtils.isEmpty(auditList) ? Constant.ONE : auditList.get(0).getSort());
    stocksAudit.setAuditStatus(StocksAuditEnum.AUDIT_STATUS_TO_BE_REVIEWED.getCode());
    stocksAudit.setTableName(getTableByType(type));
    // 添加一级审核人信息
    save(stocksAudit);
    return true;
  }

  /**
   * 获取当前记录的最新审批信息
   *
   * @param type 审批类型
   * @param id id
   * @return StocksAudit
   */
  @Override
  public StocksAudit nowAuditInfo(Long type, Long id) {
    return getOne(
        Wrappers.<StocksAudit>lambdaQuery()
            .eq(StocksAudit::getTableId, id)
            .eq(StocksAudit::getAuditStatus, StocksAuditEnum.AUDIT_STATUS_TO_BE_REVIEWED.getCode())
            .eq(StocksAudit::getTableName, getTableByType(type))
            .eq(StocksAudit::getAuditUser, SecurityUtils.getUserId())
            .eq(StocksAudit::getAuditType, type));
  }

  /**
   * 用户审批
   *
   * @param audit 审批信息
   * @return boolean
   */
  @Override
  @Transactional(rollbackFor = Exception.class)
  public boolean userApprovals(StocksAudit audit, List<MultipartFile> fileIds) {
    if (audit == null) {
      throw new ToflyDeniedException("审核信息为空");
    }

    StocksAudit stockAudit =
        getOne(
            Wrappers.<StocksAudit>lambdaQuery()
                .eq(StocksAudit::getTableId, audit.getTableId())
                .eq(
                    StocksAudit::getAuditStatus,
                    StocksAuditEnum.AUDIT_STATUS_TO_BE_REVIEWED.getCode())
                .eq(StocksAudit::getAuditType, audit.getAuditType()));

    if (!stockAudit.getAuditUser().equals(SecurityUtils.getUserId())) {
      throw new ToflyDeniedException("您无权审核该记录");
    }
    stockAudit.setAuditStatus(audit.getAuditStatus());
    stockAudit.setAuditMsg(audit.getAuditMsg());

    Long auditType = stockAudit.getAuditType();
    Long auditLevel = stockAudit.getAuditLevel();
    Long auditStatus = stockAudit.getAuditStatus();
    Long tableId = stockAudit.getTableId();
    stockAudit.setAuditTime(new Date());
    // 当前审批记录存
    updateById(stockAudit);
    // 如果审批通过
    if (auditStatus.equals(StocksAuditEnum.AUDIT_STATUS_PASS.getCode())) {
      // 查询配置表是否存在下一级审核人
      StocksAuditConfig config =
          stocksAuditConfigService.getOne(
              Wrappers.<StocksAuditConfig>lambdaQuery()
                  .eq(StocksAuditConfig::getType, auditType)
                  .eq(StocksAuditConfig::getAuditLevel, auditLevel + 1));
      // 为空则不需审核，记录审批完成
      if (config == null) {
        updateInventory(auditType, tableId);
      } else {
        // 有则继续审核生成下一条审核信息
        handleAudit(stockAudit, config);
      }
    } else if (auditStatus.equals(StocksAuditEnum.AUDIT_STATUS_HSIANG_HSIEN.getCode())) {
      // 驳回查询上一级的配置数据
      StocksAuditConfig config =
          stocksAuditConfigService.getOne(
              Wrappers.<StocksAuditConfig>lambdaQuery()
                  .eq(StocksAuditConfig::getType, auditType)
                  .eq(StocksAuditConfig::getAuditLevel, auditLevel - 1));
      reviewRejection(config, auditType, tableId);
      // 外卖审核经驳回直接驳回不返回上一级，所以不生成审核记录表
      if (!auditType.equals(StocksAuditEnum.TAKEAWAY_OUT_STOCK_TYPE.getCode()) && config != null) {
        handleAudit(stockAudit, config);
      }
    }
    return true;
  }

  // 增加审核信息
  private void handleAudit(StocksAudit audit, StocksAuditConfig config) {
    Long auditType = audit.getAuditType();
    Long tableId = audit.getTableId();
    Long auditDept = config.getAuditDept();
    Long auditUser = config.getAuditUser();
    StocksAudit stocksAudit = new StocksAudit();
    stocksAudit.setAuditLevel(config.getAuditLevel());
    stocksAudit.setAuditDept(auditDept);
    stocksAudit.setAuditUser(auditUser);
    stocksAudit.setAuditType(auditType);
    stocksAudit.setTableId(tableId);
    stocksAudit.setSort(audit.getSort() + 1);
    stocksAudit.setTableName(getTableByType(auditType));
    stocksAudit.setAuditStatus(StocksAuditEnum.AUDIT_STATUS_TO_BE_REVIEWED.getCode());
    save(stocksAudit);

    if (auditType.equals(StocksAuditEnum.IN_STOCK_CONFIG_TYPE.getCode()) || auditType.equals(StocksAuditEnum.RETURN_STOCK_CONFIG_TYPE.getCode())) {
      storageService.update(
          Wrappers.<Storage>lambdaUpdate()
              .eq(Storage::getId, tableId)
              .set(Storage::getAuditUser, auditUser)
              .set(Storage::getAuditDept, auditDept));
    } else if (auditType.equals(StocksAuditEnum.OUT_STOCK_CONFIG_TYPE.getCode()) || auditType.equals(StocksAuditEnum.ADD_STOCK_CONFIG_TYPE.getCode())) {
      pickingService.update(
          Wrappers.<Picking>lambdaUpdate()
              .eq(Picking::getId, tableId)
              .set(Picking::getAuditUser, auditUser)
              .set(Picking::getAuditDept, auditDept));
    } else if(auditType.equals(StocksAuditEnum.TAKEAWAY_OUT_STOCK_TYPE.getCode())){
      stocksTakeawayInfoService.update(
          Wrappers.<StocksTakeawayInfo>lambdaUpdate()
              .eq(StocksTakeawayInfo::getId, tableId)
              .set(StocksTakeawayInfo::getAuditUser, auditUser)
              .set(StocksTakeawayInfo::getAuditDept, auditDept));
    } else if (auditType.equals(StocksAuditEnum.PROFIT_CONFIG_TYPE.getCode())){
      profitService.update(
          Wrappers.<Profit>lambdaUpdate()
              .eq(Profit::getId, tableId)
              .set(Profit::getAuditUser, auditUser)
              .set(Profit::getAuditDept, auditDept));
    }else if (auditType.equals(StocksAuditEnum.LOSS_CONFIG_TYPE.getCode())){
      lossService.update(
          Wrappers.<Loss>lambdaUpdate()
              .eq(Loss::getId, tableId)
              .set(Loss::getAuditUser, auditUser)
              .set(Loss::getAuditDept, auditDept));
    }
  }

  /**
   * 更新库存
   *
   * @param type 类型
   * @param id 记录id
   */
  @Override
  @Transactional(rollbackFor = Exception.class)
  public void updateInventory(Long type, Long id) {
    Long approved = StocksAuditEnum.APPROVED.getCode();
    // 入库类型操作
    if (type.equals(StocksAuditEnum.IN_STOCK_CONFIG_TYPE.getCode()) || type.equals(StocksAuditEnum.RETURN_STOCK_CONFIG_TYPE.getCode())) {
      pushStorage(id);
      // 当前记录库存统计
      List<InStockCountVo> inStockCount = baseMapper.inStockInfoCount(id);
      if (CollectionUtils.isNotEmpty(inStockCount)) {
        for (InStockCountVo vo : inStockCount) {
          Double num = vo.getNum();
          // 总库存记录
          List<StocksStockInfo> stockInfo =
              stockInfoService.list(
                  Wrappers.<StocksStockInfo>lambdaQuery()
                      .eq(StocksStockInfo::getStoreId, vo.getStoreId())
                      .eq(StocksStockInfo::getDeviceId, vo.getDeviceId()));
          StocksStockInfo ssi = new StocksStockInfo();
          // 信息不为空则更新库存信息
          if (!CollectionUtils.isEmpty(stockInfo)) {
            ssi.setId(stockInfo.get(0).getId());
            ssi.setStockNum(stockInfo.get(0).getStockNum() + num);
            if (type.equals(StocksAuditEnum.IN_STOCK_CONFIG_TYPE.getCode())){
              ssi.setInstockTotalNum(stockInfo.get(0).getInstockTotalNum() + num);
            }else {
              ssi.setUnboundTotalNum(stockInfo.get(0).getUnboundTotalNum() + num);
            }
          } else {
            ssi.setDeviceId(vo.getDeviceId());
            ssi.setStoreId(vo.getStoreId());
            ssi.setStockNum(num);
            ssi.setInstockTotalNum(num);
            stockInfoService.save(ssi);
          }
          stockInfoService.saveOrUpdate(ssi);
          //入库明细保存库存id
          storageDetailService.update(Wrappers.<StorageDetail>lambdaUpdate()
            .set(StorageDetail::getInventoryId,ssi.getId())
            .eq(StorageDetail::getStorageId,id)
            .eq(StorageDetail::getItemIdCode,vo.getCode()));
        }
      }
      // 清空入库记录表保存的审核信息
      baseMapper.emptyAuditInfo(
          StocksAuditEnum.IN_STOCK_CONFIG_TABLE.getName(),
          id,
          StocksAuditEnum.IN_STOCK_CONFIG_TYPE.getCode(),
          approved);

    } else if (type.equals(StocksAuditEnum.OUT_STOCK_CONFIG_TYPE.getCode()) || type.equals(StocksAuditEnum.ADD_STOCK_CONFIG_TYPE.getCode())) {
      // 出库类型操作
      List<PickingDetail> detailList = pushPicking(id);
      if (CollectionUtils.isNotEmpty(detailList)) {
        List<StocksStockInfo> stockInfoList = new ArrayList<>();
        for (PickingDetail detail : detailList) {
          Double num = Double.valueOf(detail.getQty());
          Long stockInfoId = detail.getInventoryId();
          StocksStockInfo stockInfo = stockInfoService.getById(stockInfoId);
          if (type.equals(StocksAuditEnum.OUT_STOCK_CONFIG_TYPE.getCode())){
            stockInfo.setSupplementStockTotalNum(stockInfo.getOutstockTotalNum() + num);
          }else {
            stockInfo.setSupplementStockTotalNum(stockInfo.getOutstockTotalNum() + num);
          }
          stockInfo.setStockNum(stockInfo.getStockNum() - num);
          stockInfoList.add(stockInfo);
        }
        stockInfoService.updateBatchById(stockInfoList);
      }
      // 修改出库记录已完成审核
      baseMapper.emptyAuditInfo(
          StocksAuditEnum.OUT_STOCK_CONFIG_TABLE.getName(),
          id,
          StocksAuditEnum.OUT_STOCK_CONFIG_TYPE.getCode(),
          approved);
    } else if (type.equals(StocksAuditEnum.TAKEAWAY_OUT_STOCK_TYPE.getCode())) {
      pushTakeaway(id);
      List<StocksTakeawayDetail> detailList =
          stocksTakeawayDetailService.list(
              Wrappers.<StocksTakeawayDetail>lambdaQuery().eq(StocksTakeawayDetail::getTakeawayId, id));
      if (CollectionUtils.isNotEmpty(detailList)) {
        StocksStockInfo stockInfo;
        for (StocksTakeawayDetail takeawayDetail : detailList) {
          Double num = Double.valueOf(takeawayDetail.getQty());
          Long stockInfoId = takeawayDetail.getStockInfoId();
          stockInfo = stockInfoService.getById(stockInfoId);
          stockInfo.setTakeawayTotalNum(stockInfo.getTakeawayTotalNum() + num);
          stockInfo.setStockNum(stockInfo.getStockNum() - num);
          stockInfoService.updateById(stockInfo);
        }
      }
      baseMapper.emptyAuditInfo(
          StocksAuditEnum.TAKEAWAY_OUT_STOCK_TABLE.getName(),
          id,
          StocksAuditEnum.TAKEAWAY_OUT_STOCK_TYPE.getCode(),
          approved);
    }else if (type.equals(StocksAuditEnum.PROFIT_CONFIG_TYPE.getCode())){
      List<ProfitDetail> detailList = pushProfit(id);
      if (CollectionUtils.isNotEmpty(detailList)) {
        List<StocksStockInfo> stockInfoList = new ArrayList<>();
        for (ProfitDetail detail : detailList) {
          Double num = detail.getQty();
          Long stockInfoId = detail.getInventoryId();
          StocksStockInfo stockInfo = stockInfoService.getById(stockInfoId);
          stockInfo.setProfitTotalNum(num);
          stockInfo.setStockNum(detail.getQtyActual());
          stockInfoList.add(stockInfo);
        }
        stockInfoService.updateBatchById(stockInfoList);
      }
      // 修改出库记录已完成审核
      baseMapper.emptyAuditInfo(
              StocksAuditEnum.PROFIT_CONFIG_TABLE.getName(),
              id,
              StocksAuditEnum.PROFIT_CONFIG_TYPE.getCode(),
              approved);
    }else if (type.equals(StocksAuditEnum.LOSS_CONFIG_TYPE.getCode())){
      List<LossDetail> detailList = pushLoss(id);
      if (CollectionUtils.isNotEmpty(detailList)) {
        List<StocksStockInfo> stockInfoList = new ArrayList<>();
        for (LossDetail detail : detailList) {
          Double num = detail.getQty();
          Long stockInfoId = detail.getInventoryId();
          StocksStockInfo stockInfo = stockInfoService.getById(stockInfoId);
          stockInfo.setLossTotalNum(num);
          stockInfo.setStockNum(detail.getQtyActual());
          stockInfoList.add(stockInfo);
        }
        stockInfoService.updateBatchById(stockInfoList);
      }
      // 修改出库记录已完成审核
      baseMapper.emptyAuditInfo(
              StocksAuditEnum.LOSS_CONFIG_TABLE.getName(),
              id,
              StocksAuditEnum.LOSS_CONFIG_TYPE.getCode(),
              approved);
    }
  }

  /**
   * 审核驳回
   *
   * @param type 类型
   * @param config 配置信息
   * @param id id
   */
  @Override
  public void reviewRejection(StocksAuditConfig config, Long type, Long id) {
    Long singHsien = StocksAuditEnum.HSIANG_HSIEN.getCode();

    if (type.equals(StocksAuditEnum.IN_STOCK_CONFIG_TYPE.getCode()) || type.equals(StocksAuditEnum.RETURN_STOCK_CONFIG_TYPE.getCode())) {
      if (config == null) {
        // 重置入库未未审核状态
        baseMapper.emptyAuditInfo(
            StocksAuditEnum.IN_STOCK_CONFIG_TABLE.getName(),
            id,
            StocksAuditEnum.IN_STOCK_CONFIG_TYPE.getCode(),
            singHsien);
      } else {
        // 修改入库审核信息
        storageService.update(
            Wrappers.<Storage>lambdaUpdate()
                .eq(Storage::getId, id)
                .set(Storage::getAuditUser, config.getAuditUser())
                .set(Storage::getAuditDept, config.getAuditDept()));
      }
    } else if (type.equals(StocksAuditEnum.OUT_STOCK_CONFIG_TYPE.getCode()) || type.equals(StocksAuditEnum.ADD_STOCK_CONFIG_TYPE.getCode())) {
      if (config == null) {
        // 重置出库未审核状态
        baseMapper.emptyAuditInfo(
            StocksAuditEnum.OUT_STOCK_CONFIG_TABLE.getName(),
            id,
            StocksAuditEnum.OUT_STOCK_CONFIG_TYPE.getCode(),
            singHsien);
      } else {
        // 修改入库审核信息
        pickingService.update(
            Wrappers.<Picking>lambdaUpdate()
                .eq(Picking::getId, id)
                .set(Picking::getAuditUser, config.getAuditUser())
                .set(Picking::getAuditDept, config.getAuditDept()));
      }
    } else if (type.equals(StocksAuditEnum.TAKEAWAY_OUT_STOCK_TYPE.getCode())) {
      baseMapper.emptyAuditInfo(
          StocksAuditEnum.TAKEAWAY_OUT_STOCK_TABLE.getName(),
          id,
          StocksAuditEnum.TAKEAWAY_OUT_STOCK_TYPE.getCode(),
          singHsien);
    }else if (type.equals(StocksAuditEnum.PROFIT_CONFIG_TYPE.getCode())){
      if (config == null) {
        // 重置入库未未审核状态
        baseMapper.emptyAuditInfo(
                StocksAuditEnum.PROFIT_CONFIG_TABLE.getName(),
                id,
                StocksAuditEnum.PROFIT_CONFIG_TYPE.getCode(),
                singHsien);
      } else {
        // 修改入库审核信息
        profitService.update(
                Wrappers.<Profit>lambdaUpdate()
                        .eq(Profit::getId, id)
                        .set(Profit::getAuditUser, config.getAuditUser())
                        .set(Profit::getAuditDept, config.getAuditDept()));
      }
    }else if (type.equals(StocksAuditEnum.LOSS_CONFIG_TYPE.getCode())){
      if (config == null) {
        // 重置入库未未审核状态
        baseMapper.emptyAuditInfo(
                StocksAuditEnum.LOSS_CONFIG_TABLE.getName(),
                id,
                StocksAuditEnum.LOSS_CONFIG_TYPE.getCode(),
                singHsien);
      } else {
        // 修改入库审核信息
        lossService.update(
                Wrappers.<Loss>lambdaUpdate()
                        .eq(Loss::getId, id)
                        .set(Loss::getAuditUser, config.getAuditUser())
                        .set(Loss::getAuditDept, config.getAuditDept()));
      }
    }
  }

  @Override
  public boolean testPush(Long id) {
    pushStorage(id);
    return true;
  }

  /**
   * 入库推送
   * @param id id
   */
  private void pushStorage(Long id){
    Storage storage = storageService.getById(id);
    List<StorageDetail> detailList = storage.getDetailList();

    List<StorageDetailParamVo> detailVoList = new ArrayList<>();
    //格式转换
    for (StorageDetail storageDetail : detailList) {
      StorageDetailParamVo vo = new StorageDetailParamVo();
      BeanUtils.copyProperties(storageDetail,vo);
      vo.setNote(StringUtils.isNotBlank(storageDetail.getNote()) ? storageDetail.getNote() : "");
      detailVoList.add(vo);
    }
    StorageParamVo storageParamVo = new StorageParamVo();
    storage.setDetailList(null);
    BeanUtils.copyProperties(storage,storageParamVo);
    storageParamVo.setDetailList(detailVoList);
    storageParamVo.setFDate(DateUtil.format(storage.getFDate(),"yyyy-MM-dd HH:mm:ss"));
    storageParamVo.setBillerIdName("Administrator");
    //body信息
    String body = JSONObject.toJSONString(storageParamVo);
    //获取token信息
    String tokenInfo = callApiService.getToken();
    //获取url地址
    String url = JinDieUrlEnum.SAVE_UPDATE_STORAGE_URL.getName() + "?token=" + tokenInfo;
    //入库单登记推送
    callApiService.saveResult(body, url);
  }

  private void pushTakeaway(Long id){
    // 外卖出库推送
    StocksTakeawayInfo takeaway = stocksTakeawayInfoService.getById(id);
    List<StocksTakeawayDetail> infoList = takeaway.getDetailList();
    //金蝶数据推送
    List<PickingDetailParamVo> detailVoList = new ArrayList<>();
    //格式转换
    for (StocksTakeawayDetail detail : infoList) {
      PickingDetailParamVo vo = new PickingDetailParamVo();
      BeanUtils.copyProperties(detail,vo);
      vo.setNote(StringUtils.isNotBlank(detail.getNote()) ? detail.getNote() : "");
      detailVoList.add(vo);
    }
    PickingParamVo pickingParam = new PickingParamVo();
    takeaway.setDetailList(null);
    BeanUtils.copyProperties(takeaway,pickingParam);
    pickingParam.setDetailList(detailVoList);
    pickingParam.setFDate(DateUtil.format(takeaway.getFDate(),"yyyy-MM-dd HH:mm:ss"));
    //body信息
    String body = JSONObject.toJSONString(pickingParam);
    //获取token信息
    String tokenInfo = callApiService.getToken();
    //获取url地址
    String url = JinDieUrlEnum.SAVE_UPDATE_PICKING_URL.getName() + "?token=" + tokenInfo;
    callApiService.saveResult(body, url);
  }

  /**
   * 出库推送
   * @param id id
   * @return list
   */
  private List<PickingDetail> pushPicking(Long id){
    // 出库类型操作
    Picking picking = pickingService.getById(id);
    List<PickingDetail> detailList = picking.getDetailList();
    //金蝶数据推送
    List<PickingDetailParamVo> detailVoList = new ArrayList<>();
    //格式转换
    for (PickingDetail pickingDetail : detailList) {
      PickingDetailParamVo vo = new PickingDetailParamVo();
      BeanUtils.copyProperties(pickingDetail,vo);
      vo.setNote(StringUtils.isNotBlank(pickingDetail.getNote()) ? pickingDetail.getNote() : "");
      detailVoList.add(vo);
    }
    PickingParamVo pickingParam = new PickingParamVo();
    picking.setDetailList(null);
    BeanUtils.copyProperties(picking,pickingParam);
    pickingParam.setDetailList(detailVoList);
    pickingParam.setFDate(DateUtil.format(picking.getFDate(),"yyyy-MM-dd HH:mm:ss"));
    //body信息
    String body = JSONObject.toJSONString(pickingParam);
    //获取token信息
    String tokenInfo = callApiService.getToken();
    //获取url地址
    String url = JinDieUrlEnum.SAVE_UPDATE_PICKING_URL.getName() + "?token=" + tokenInfo;
    callApiService.saveResult(body, url);
    return detailList;
  }

  private List<ProfitDetail> pushProfit(Long id){
    Profit profit = profitService.getById(id);
    List<ProfitDetail> detailList = profit.getDetailList();
    List<ProfitDetailParamVo> detailVoList = new ArrayList<>();
    //格式转换
    for (ProfitDetail profitDetail : detailList) {
      ProfitDetailParamVo vo = new ProfitDetailParamVo();
      BeanUtils.copyProperties(profitDetail,vo);
      vo.setNote(StringUtils.isNotBlank(profitDetail.getNote()) ? profitDetail.getNote() : "");
      detailVoList.add(vo);
    }
    ProfitParamVo profitParamVo = new ProfitParamVo();
    profit.setDetailList(null);
    BeanUtils.copyProperties(profit,profitParamVo);
    profitParamVo.setDetailList(detailVoList);
    profitParamVo.setFDate(DateUtil.format(profit.getFDate(),"yyyy-MM-dd HH:mm:ss"));
    //body信息
    String body = JSONObject.toJSONString(profitParamVo);
    //获取token信息
    String tokenInfo = callApiService.getToken();
    //获取url地址
    String url = JinDieUrlEnum.SAVE_UPDATE_PROFIT_URL.getName() + "?token=" + tokenInfo;
    callApiService.saveResult(body, url);

    return detailList;
  }

  private List<LossDetail> pushLoss(Long id){
    Loss loss = lossService.getById(id);
    List<LossDetail> detailList = loss.getDetailList();
    List<LossDetailParamVo> detailVoList = new ArrayList<>();
    //格式转换
    for (LossDetail lossDetail : detailList) {
      LossDetailParamVo vo = new LossDetailParamVo();
      BeanUtils.copyProperties(lossDetail,vo);
      vo.setNote(StringUtils.isNotBlank(lossDetail.getNote()) ? lossDetail.getNote() : "");
      detailVoList.add(vo);
    }
    LossParamVo lossParamVo = new LossParamVo();
    loss.setDetailList(null);
    BeanUtils.copyProperties(loss,lossParamVo);
    lossParamVo.setDetailList(detailVoList);
    lossParamVo.setFDate(DateUtil.format(loss.getFDate(),"yyyy-MM-dd HH:mm:ss"));
    //body信息
    String body = JSONObject.toJSONString(lossParamVo);
    //获取token信息
    String tokenInfo = callApiService.getToken();
    //获取url地址
    String url = JinDieUrlEnum.SAVE_UPDATE_LOSS_URL.getName() + "?token=" + tokenInfo;
    callApiService.saveResult(body, url);
    return detailList;
  }

  public static String getTableByType(Long type) {
    String tableName = null;
    if (type.equals(StocksAuditEnum.IN_STOCK_CONFIG_TYPE.getCode()) || type.equals(StocksAuditEnum.RETURN_STOCK_CONFIG_TYPE.getCode())) {
      tableName = StocksAuditEnum.IN_STOCK_CONFIG_TABLE.getName();
    } else if (type.equals(StocksAuditEnum.OUT_STOCK_CONFIG_TYPE.getCode()) || type.equals(StocksAuditEnum.ADD_STOCK_CONFIG_TYPE.getCode())) {
      tableName = StocksAuditEnum.OUT_STOCK_CONFIG_TABLE.getName();
    } else if (type.equals(StocksAuditEnum.TAKEAWAY_OUT_STOCK_TYPE.getCode())) {
      tableName = StocksAuditEnum.TAKEAWAY_OUT_STOCK_TABLE.getName();
    }else if (type.equals(StocksAuditEnum.PROFIT_CONFIG_TYPE.getCode())){
      tableName = StocksAuditEnum.PROFIT_CONFIG_TABLE.getName();
    }else if (type.equals(StocksAuditEnum.LOSS_CONFIG_TYPE.getCode())){
      tableName = StocksAuditEnum.LOSS_CONFIG_TABLE.getName();
    }
    return tableName;
  }
}
