/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.yswzMaterials.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.core.util.io.ExcelUtil;
import com.tofly.pojo.Syschildcode;
import com.tofly.yswzMaterials.entity.StocksDeviceInfo;
import com.tofly.yswzMaterials.entity.StocksMaterialType;
import com.tofly.yswzMaterials.entity.StocksStockInfo;
import com.tofly.yswzMaterials.entity.dto.DeviceInfoQuery;
import com.tofly.yswzMaterials.entity.dto.DeviceInfoStoreQuery;
import com.tofly.yswzMaterials.entity.vo.*;
import com.tofly.yswzMaterials.mapper.StocksDeviceInfoMapper;
import com.tofly.yswzMaterials.service.StocksDeviceInfoService;
import com.tofly.yswzMaterials.service.StocksMaterialTypeService;
import com.tofly.yswzMaterials.service.StocksStockInfoService;
import lombok.AllArgsConstructor;
import org.apache.commons.lang.time.DateFormatUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 物资设备信息表（包含物料、设备、药品）
 *
 * @author JF
 * @date Wed Mar 29 00:00:00 CST 2023
 */
@Service("tfStocksDeviceInfoService")
@AllArgsConstructor
public class StocksDeviceInfoServiceImpl
    extends ServiceImpl<StocksDeviceInfoMapper, StocksDeviceInfo>
    implements StocksDeviceInfoService {
  private final StocksStockInfoService stockInfoService;

  private final StocksMaterialTypeService materialTypeService;

  /**
   * 检查名称是否存在，用于修改和新增
   *
   * @param deviceInfo
   * @return
   */
  public boolean isExist(StocksDeviceInfo deviceInfo) {

    QueryWrapper<StocksDeviceInfo> queryWrapper = new QueryWrapper<>();
    queryWrapper.eq("DEVICE_NAME", deviceInfo.getDeviceName());
    queryWrapper.eq("DEVICE_TYPE", deviceInfo.getDeviceType());

    if (deviceInfo.getId() != null && deviceInfo.getId() > 0) {
      queryWrapper.ne("ID", deviceInfo.getId());
    }
    Integer count = baseMapper.selectCount(queryWrapper);

    if (count > 0) {
      return true;
    }
    return false;
  }

  @Override
  public boolean saveDeviceInfo(StocksDeviceInfo deviceInfo) {
    /*if (isExist(deviceInfo)) {
      throw new RuntimeException("名称已存在，不能新增");
    }
    String baseNo = "SBBM";
    if (deviceInfo.getDeviceType() == 62001) {
      baseNo = "WLBM";
    } else if (deviceInfo.getDeviceType() == 62003) {
      baseNo = "YPBM";
    }
    Date now = new Date(); // 创建一个Date对象，获取当前时间
    String strDateFormat = "yyyyMMdd";
    String start_no = baseNo + DateFormatUtils.format(now, strDateFormat);
    // 获取当天最大的采样编号
    StocksDeviceInfo deviceInfoMax =
        this.baseMapper
            .selectList(
                Wrappers.<StocksDeviceInfo>lambdaQuery()
                    .like(StocksDeviceInfo::getDeviceNo, start_no)
                    .orderByDesc(StocksDeviceInfo::getDeviceNo))
            .stream()
            .findFirst()
            .orElse(new StocksDeviceInfo());

    String Sample_No = "";
    if (StringUtil.isNotEmpty(deviceInfoMax.getDeviceNo())) {
      Sample_No = deviceInfoMax.getDeviceNo();
    }

    int firstNum = 1;
    if (Sample_No.length() == 16) {
      firstNum = Integer.parseInt(Sample_No.substring(13)) + 1;
    }

    String end_no = String.format("%04d", firstNum); // 补全数字，不足用0补位

    String deviceNo = start_no + end_no;
    deviceInfo.setDeviceNo(deviceNo);*/

    return this.save(deviceInfo);
  }

  @Override
  public boolean editDeviceInfo(StocksDeviceInfo deviceInfo) {
    if (isExist(deviceInfo)) {
      throw new RuntimeException("名称已存在，不能修改");
    }
    return this.updateById(deviceInfo);
  }

  @Override
  public Page<DeviceInfoStoreVo> getDeviceStorelistPage(
      Page page, DeviceInfoStoreQuery deviceInfoStoreQuery) {
    return this.baseMapper.getDeviceStorelistPage(page, deviceInfoStoreQuery);
  }

  @Override
  @Transactional(rollbackFor = Exception.class)
  public boolean deleteByIds(String ids) {
    List<String> lstIds = Arrays.asList(ids.split(","));

    QueryWrapper<StocksStockInfo> queryWrapper = new QueryWrapper<>();
    queryWrapper.in("DEVICE_ID", lstIds);
    Integer count = stockInfoService.count(queryWrapper);
    if (count > 0) {
      throw new RuntimeException("选择的设备已有库存记录，不能删除");
    }

    return this.removeByIds(lstIds);
  }

  //    @Override
  //    public void DeviceExport(StocksDeviceInfo deviceInfo) {
  //
  //        List<DeviceInfoExportVo> list = this.baseMapper.selectList(Wrappers.query(deviceInfo)).
  //                stream().map(deviceInfo_e -> {
  //                    DeviceInfoExportVo deviceInfoExportVo = new DeviceInfoExportVo();
  //                    BeanUtils.copyProperties(deviceInfo_e, deviceInfoExportVo);
  //                    return deviceInfoExportVo;
  //                }).collect(Collectors.toList());
  //
  //        ExcelUtils.getListToExcel(DeviceInfoExportVo.class, list, "类型导出");
  //
  //    }

  @Override
  @Transactional(rollbackFor = Exception.class)
  public boolean ImportData(MultipartFile file) {

    if (file == null || file.isEmpty()) {
      throw new RuntimeException("导入文件不能为空");
    }

    List<DeviceInfoExportVo> dtos = null;
    try {
      dtos = getExeData(file);
    } catch (Exception e) {
      throw new RuntimeException("文件异常");
    }
    if (CollUtil.isEmpty(dtos)) {
      throw new RuntimeException("数据不存在");
    }
    // 从字典里面获取对应的类别id
    List<Syschildcode> listDis = baseMapper.getDicByPcId(62000L); // 物资类型id,固定
    if (listDis == null || listDis.stream().count() == 0) {
      throw new RuntimeException("物资类型字典不存在，请添加");
    }
    for (DeviceInfoExportVo deviceInfoExportVo : dtos) {

      Syschildcode syschildcode =
          listDis.stream()
              .filter(
                  a ->
                      a.getCname().equals(deviceInfoExportVo.getDeviceTypeName())
                          && "1".equals(a.getCdisable()))
              .findAny()
              .orElse(null);
      if (syschildcode == null) {
        throw new RuntimeException("类别：[" + deviceInfoExportVo.getDeviceTypeName() + "]不存在，请添加字典");
      }
      Long deviceType = syschildcode.getId();

      // 根据类别id和类型名称获取类型id
      Long materialTypeId =
          materialTypeService
              .getOne(
                  Wrappers.<StocksMaterialType>lambdaQuery()
                      .eq(StocksMaterialType::getMaterialCategoryId, deviceType)
                      .eq(
                          StocksMaterialType::getMaterialTypeName,
                          deviceInfoExportVo.getMaterialTypeName()))
              .getId();
      if (materialTypeId == null || materialTypeId == 0) {
        throw new RuntimeException(
            "类别:["
                + deviceInfoExportVo.getDeviceTypeName()
                + "]下类型：["
                + deviceInfoExportVo.getMaterialTypeName()
                + "]不存在，请先添加对应的类型");
      }
      StocksDeviceInfo deviceInfo = new StocksDeviceInfo();
      BeanUtils.copyProperties(deviceInfoExportVo, deviceInfo);
      deviceInfo.setDeviceType(deviceType);
      deviceInfo.setMaterialTypeId(materialTypeId);
      // 名称存在了，直接跳过
      if (!isExist(deviceInfo)) {
        this.save(deviceInfo);
      }
    }

    return true;
  }

  @Override
  public Page<DeviceRepairVo> getDeviceRepairListPage(Page page, Long deviceId) {
    Page<DeviceRepairVo> list = new Page<>();
    SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
    Date d = new Date();
    System.out.println("今天的日期：" + df.format(d));
    List<DeviceRepairVo> list_m = new ArrayList<>();
    for (int i = 1; i < 11; i++) {
      DeviceRepairVo vo = new DeviceRepairVo();
      Random random = new Random();
      vo.setRepairCount(random.nextInt(20));
      vo.setRepairTypeName("维修类型");
      vo.setDispatchUser("张三" + i);
      vo.setDispatchState("已维修");
      vo.setDispatchTime(new Date());
      vo.setRepairStartTime(new Date(d.getTime() - i * 24 * 60 * 60 * 1000));
      vo.setRepairEndTime(new Date(vo.getRepairStartTime().getTime() + 10 * 60 * 60 * 1000));

      list_m.add(vo);
    }
    list.setRecords(list_m);

    list.setTotal(list_m.size());
    return list;
  }

  @Override
  public Page<DeviceMaintainVo> getDeviceMaintainListPage(Page page, Long deviceId) {
    Page<DeviceMaintainVo> list = new Page<>();
    List<DeviceMaintainVo> listM = new ArrayList<>();
    Date d = new Date();
    for (int i = 1; i < 11; i++) {

      DeviceMaintainVo vo = new DeviceMaintainVo();
      vo.setMaintainTypeName("保养类型" + i);

      vo.setDispatchUser("张三" + i);
      vo.setDispatchState("已维修");
      vo.setDispatchTime(new Date());
      vo.setMaintainStartTime(new Date(d.getTime() - i * 24 * 60 * 60 * 1000));
      vo.setMaintainEndTime(new Date(vo.getMaintainStartTime().getTime() + 10 * 60 * 60 * 1000));

      listM.add(vo);
    }
    list.setRecords(listM);
    list.setTotal(listM.size());

    return list;
  }

  @Override
  public Page<DevicePatrolVo> getDevicePatrolListPage(Page page, Long deviceId) {
    Date d = new Date();
    Page<DevicePatrolVo> list = new Page<>();
    List<DevicePatrolVo> list_m = new ArrayList<>();
    for (int i = 1; i < 11; i++) {

      DevicePatrolVo vo = new DevicePatrolVo();
      vo.setPatrolCategory("巡检类别" + i);
      vo.setPatrolType("巡检类型" + i);
      vo.setPatrolUser("张三" + i);
      vo.setPatrolStartTime(new Date(d.getTime() - i * 24 * 60 * 60 * 1000));
      vo.setPatrolEndTime(new Date(vo.getPatrolStartTime().getTime() + 10 * 60 * 60 * 1000));

      vo.setPatrolState("巡检状态" + i);
      list_m.add(vo);
    }
    list.setRecords(list_m);
    list.setTotal(list_m.stream().count());
    return list;
  }

  @Override
  public DeviceReportVo getDeviceReport(Long deviceId) {
    DeviceReportVo deviceReportVo = new DeviceReportVo();
    Page page = new Page();
    page.setSize(1);
    page.setCurrent(1);

    deviceReportVo.setRepairCount(this.getDeviceRepairListPage(page, deviceId).getTotal());
    deviceReportVo.setMaintainCount(this.getDeviceMaintainListPage(page, deviceId).getTotal());
    deviceReportVo.setPatrolCount(this.getDevicePatrolListPage(page, deviceId).getTotal());

    return deviceReportVo;
  }

  @Override
  public Page<StocksDeviceInfo> getDataPage(Page page, DeviceInfoQuery query) {
    return this.baseMapper.getDataPage(page, query);
  }

  @Override
  public void deleteInfoByNumber(Set<String> number) {
    baseMapper.deleteInfoByNumber(number);
  }

  /**
   * 批量添加物料信息
   *
   * @param info info
   * @return boolean
   */
  @Override
  public boolean batchInsertInfo(List<StocksDeviceInfo> info) {
    return baseMapper.batchInsertInfo(info);
  }

  /**
   * 获取设备单位组
   *
   * @return List<String>
   */
  @Override
  public List<String> getDeviceUnitGroup() {
    return baseMapper.getDeviceUnitGroup();
  }

  /**
   * 文件转换为对象数据
   *
   * @param fileList 上传的文件
   * @return List<DispatchInfoDto>
   * @throws Exception
   */
  private List<DeviceInfoExportVo> getExeData(MultipartFile fileList) throws Exception {
    InputStream inputStream = fileList.getInputStream();
    ExcelUtil opsExcel = new ExcelUtil(DeviceInfoExportVo.class);
    List<DeviceInfoExportVo> ips = opsExcel.importExcel("", inputStream);
    return ips;
  }
}
