/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.yswzMaterials.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.yswzMaterials.em.StocksAuditEnum;
import com.tofly.yswzMaterials.entity.StocksAudit;
import com.tofly.yswzMaterials.entity.StocksOutstockDetail;
import com.tofly.yswzMaterials.entity.StocksOutstockInfo;
import com.tofly.yswzMaterials.entity.StocksStockInfo;
import com.tofly.yswzMaterials.entity.dto.OutstockInfoDto;
import com.tofly.yswzMaterials.entity.dto.OutstockInfoQuery;
import com.tofly.yswzMaterials.entity.vo.OutstockDetailVo;
import com.tofly.yswzMaterials.entity.vo.OutstockInfoVo;
import com.tofly.yswzMaterials.mapper.StocksOutstockInfoMapper;
import com.tofly.yswzMaterials.service.*;
import org.apache.commons.lang.time.DateFormatUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 库存出库信息表
 *
 * @author JF
 * @date Tue Dec 13 00:00:00 CST 2022
 */
@Service("tfStocksOutstockInfoService")
public class StocksOutstockInfoServiceImpl
    extends ServiceImpl<StocksOutstockInfoMapper, StocksOutstockInfo>
    implements StocksOutstockInfoService {

  @Autowired private IFileInfoService fileInfoService;

  @Value("${yswz.ftp.stocksOutstockInfo}")
  private String path;

  @Autowired private StocksOutstockDetailService outstockDetailService;

  @Autowired private StocksStockInfoService stockInfoService;

  @Autowired private StocksAuditService stocksAuditService;

  private String getRkType(Long type) {
    String str = "";
    if (type == 1) {
      str = "WZCK";
    } else if (type == 2) {
      str = "SBCK";
    } else if (type == 3) {
      str = "YPCK";
    }
    if (StringUtil.isEmpty(str)) {
      throw new RuntimeException("出库类型错误");
    }
    return str;
  }

  @Override
  @Transactional(rollbackFor = Exception.class)
  public boolean saveOutstockInfo(OutstockInfoDto outstockInfoDto, List<MultipartFile> fileIds) {
    boolean flag;
    if (CollectionUtils.isEmpty(outstockInfoDto.getOutstockDetails())) {
      throw new RuntimeException("入库明细单数据不能为空");
    }
    String baseNo = getRkType(outstockInfoDto.getOutstockCategory());
    // 创建一个Date对象，获取当前时间
    Date now = new Date();
    String strDateFormat = "yyyyMM";
    String start_no = baseNo + DateFormatUtils.format(now, strDateFormat);
    // 获取当天最大的采样编号
    String Sample_No = this.baseMapper.getOutstockMaxNo(start_no);

    int firstNum = 1;
    if (Sample_No.length() == 14) {
      firstNum = Integer.parseInt(Sample_No.substring(11)) + 1;
    }
    // 补全数字，不足用0补位
    String end_no = String.format("%04d", firstNum);

    String outstockNo = start_no + end_no;

    StocksOutstockInfo data = new StocksOutstockInfo();
    BeanUtils.copyProperties(outstockInfoDto, data);
    // 赋值单号
    data.setOutstockNo(outstockNo);
    // 默认入库人为当前登录人
    data.setOutstockUserId(SecurityUtils.getUserId());
    data.setOutstockDpt(SecurityUtils.getUserDeptId());

    // 附件
    data.setAnnex(
        fileInfoService.uploadFiles(
            fileIds, data.getAnnex(), data.getAnnex(), path, "TF_STOCKS_OUTSTOCK_INFO"));

    data.setOutstockStatus(StocksAuditEnum.NOT_REVIEWED.getCode());
    flag = this.save(data);
    // 保存入库明细数据
    for (StocksOutstockDetail outstockDetail : outstockInfoDto.getOutstockDetails()) {
      outstockDetail.setOutstockInfoId(data.getId());
      outstockDetail.setId(null);
      outstockDetailService.save(outstockDetail);
    }
    /*    if (data.getOutstockStatus() == 2) { // 已确认的
      AddStockInfo(outstockInfoDto.getOutstockDetails()); // 加入库存
    }*/
    return flag;
  }

  @Override
  @Transactional(rollbackFor = Exception.class)
  public boolean editOutstockInfo(OutstockInfoDto outstockInfoDto, List<MultipartFile> fileIds) {
    boolean flag;
    StocksOutstockInfo outstockInfo = this.getById(outstockInfoDto.getId());
    if (outstockInfo == null) {
      throw new RuntimeException("修改，参数Id错误");
    }
    if (!outstockInfoDto.getOutstockUserId().equals(SecurityUtils.getUserId())) {
      throw new RuntimeException("只能修改自己创建的数据");
    }
    outstockInfoDto.setOutstockNo(outstockInfo.getOutstockNo());
    StocksOutstockInfo data = new StocksOutstockInfo();
    BeanUtils.copyProperties(outstockInfoDto, data);

    data.setAnnex(
        fileInfoService.uploadFiles(
            fileIds, data.getAnnex(), outstockInfo.getAnnex(), path, "TF_STOCKS_OUTSTOCK_INFO"));

    flag = this.updateById(data);

    outstockDetailService.remove(
        Wrappers.<StocksOutstockDetail>lambdaQuery()
            .eq(StocksOutstockDetail::getOutstockInfoId, data.getId()));

    for (StocksOutstockDetail outstockDetail : outstockInfoDto.getOutstockDetails()) {
      outstockDetail.setOutstockInfoId(data.getId());
      outstockDetailService.save(outstockDetail);
    }
    return flag;
  }

  /**
   * 出库
   *
   * @param outstockDetails
   */
  private void AddStockInfo(List<OutstockDetailVo> outstockDetails) {
    for (StocksOutstockDetail detail : outstockDetails) {
      StocksStockInfo existStockInfo = stockInfoService.getById(detail.getStockInfoId());
      if (null == existStockInfo) {
        throw new RuntimeException("所选设备【" + detail.getDeviceName() + "】没有对应的库存，不能出库");
      }
      if (existStockInfo.getStockNum() < detail.getOutstockNum()) {
        throw new RuntimeException(
            "所选设备【"
                + detail.getDeviceName()
                + "】出库数量:"
                + detail.getOutstockNum()
                + "，不能大于当前库存总数:"
                + existStockInfo.getStockNum());
      }
      StocksStockInfo stockInfo = new StocksStockInfo();
      stockInfo.setId(existStockInfo.getId());
      stockInfo.setOutstockTotalNum(existStockInfo.getOutstockTotalNum() + detail.getOutstockNum());
      stockInfo.setStockNum(existStockInfo.getStockNum() - detail.getOutstockNum());
      stockInfoService.updateById(stockInfo);
    }
  }

  @Override
  @Transactional(rollbackFor = Exception.class)
  public boolean deleteByIds(String ids) {
    List<String> lstIds = Arrays.asList(ids.split(","));
    int count =
        this.count(
            Wrappers.<StocksOutstockInfo>lambdaQuery()
                .in(StocksOutstockInfo::getId, ids)
                .and(
                    a ->
                        a.eq(
                                StocksOutstockInfo::getOutstockStatus,
                                StocksAuditEnum.UNDER_REVIEW.getCode())
                            .or()
                            .eq(
                                StocksOutstockInfo::getOutstockStatus,
                                StocksAuditEnum.APPROVED.getCode())));

    if (count > 0) {
      throw new RuntimeException("存在正在审核或审核完成的记录，无法删除");
    }
    //删除审核信息
    stocksAuditService.remove(
        Wrappers.<StocksAudit>lambdaQuery()
            .eq(StocksAudit::getAuditType, StocksAuditEnum.OUT_STOCK_CONFIG_TYPE.getCode())
            .in(StocksAudit::getTableId, lstIds));
    // 删除明细
    outstockDetailService.remove(
        Wrappers.<StocksOutstockDetail>lambdaQuery()
            .in(StocksOutstockDetail::getOutstockInfoId, ids));
    // 删除主表
    return this.removeByIds(lstIds);
  }

  @Override
  public Page<OutstockInfoVo> getDataPage(Page page, OutstockInfoQuery outstockInfoQuery) {
    QueryWrapper<StocksOutstockInfo> queryWrapper = getOutstockInfoQueryWrapper(outstockInfoQuery);
    return page(page, queryWrapper);
  }

  /**
   * 获取审核完成的出库编号
   *
   * @return list
   */
  @Override
  public List<Map<String, Object>> getOutStockCode() {
    return baseMapper.getOutStockCode();
  }

  /**
   * 查询条件拼接
   *
   * @param outstockInfoQuery
   * @return
   */
  private QueryWrapper<StocksOutstockInfo> getOutstockInfoQueryWrapper(
      OutstockInfoQuery outstockInfoQuery) {
    QueryWrapper<StocksOutstockInfo> queryWrapper = new QueryWrapper<>();

    if (outstockInfoQuery.getOutstockCategory() != null
        && outstockInfoQuery.getOutstockCategory() > 0) {

      queryWrapper.eq("OUTSTOCK_CATEGORY", outstockInfoQuery.getOutstockCategory());
    }

    if (outstockInfoQuery.getOutstockTimeStart() != null) {


      queryWrapper.ge("OUTSTOCK_TIME", outstockInfoQuery.getOutstockTimeStart());
    }
    if (outstockInfoQuery.getOutstockTimeEnd() != null) {

      queryWrapper.le("OUTSTOCK_TIME", outstockInfoQuery.getOutstockTimeEnd());
    }

    if (StringUtil.isNotEmpty(outstockInfoQuery.getOutstockNo())) {

      queryWrapper.like("OUTSTOCK_NO", outstockInfoQuery.getOutstockNo());
    }

    if (outstockInfoQuery.getStoreId() != null && outstockInfoQuery.getStoreId() > 0) {

      queryWrapper.eq("STORE_ID", outstockInfoQuery.getStoreId());
    }
    if (outstockInfoQuery.getWaterworksId() != null && outstockInfoQuery.getWaterworksId() > 0) {

      queryWrapper.eq("FIRM", outstockInfoQuery.getWaterworksId());
    }

    if (outstockInfoQuery.getOutstockTypeId() != null
        && outstockInfoQuery.getOutstockTypeId() > 0) {

      queryWrapper.eq("OUTSTOCK_TYPE_ID", outstockInfoQuery.getOutstockTypeId());
    }
    if (outstockInfoQuery.getBuninessType() != null && outstockInfoQuery.getBuninessType() > 0) {

      queryWrapper.eq("BUNINESS_TYPE", outstockInfoQuery.getBuninessType());
    }

    if (outstockInfoQuery.getOutstockStatus() != null
        && outstockInfoQuery.getOutstockStatus() > 0) {

      queryWrapper.eq("OUTSTOCK_STATUS", outstockInfoQuery.getOutstockStatus());
    }
    queryWrapper.orderByDesc("CREATE_TIME");

    return queryWrapper;
  }
}
