package com.tofly.yswzMaterials.service.impl;

import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.yswzMaterials.em.StocksAuditEnum;
import com.tofly.yswzMaterials.entity.StocksStockInfo;
import com.tofly.yswzMaterials.entity.StocksSupplementDetail;
import com.tofly.yswzMaterials.entity.StocksSupplementInfo;
import com.tofly.yswzMaterials.entity.dto.SupplementInfoDto;
import com.tofly.yswzMaterials.entity.dto.SupplementInfoQuery;
import com.tofly.yswzMaterials.entity.vo.OutstockInfoVo;
import com.tofly.yswzMaterials.mapper.StocksSupplementInfoMapper;
import com.tofly.yswzMaterials.service.IFileInfoService;
import com.tofly.yswzMaterials.service.StocksStockInfoService;
import com.tofly.yswzMaterials.service.StocksSupplementDetailService;
import com.tofly.yswzMaterials.service.StocksSupplementInfoService;
import org.apache.commons.lang.time.DateFormatUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author bubble
 */
@Service
public class StocksSupplementInfoServiceImpl
    extends ServiceImpl<StocksSupplementInfoMapper, StocksSupplementInfo>
    implements StocksSupplementInfoService {

  @Autowired private IFileInfoService fileInfoService;
  @Autowired private StocksSupplementDetailService stocksSupplementDetailService;
  @Autowired private StocksStockInfoService stocksStockInfoService;

  @Value("${yswz.ftp.supplementInfo}")
  private String path;

  /**
   * 保存增补出库信息
   *
   * @param infoDto dto
   * @param fileIds 文件
   * @return boolean
   */
  @Override
  @Transactional(rollbackFor = Exception.class)
  public boolean saveSupplementInfo(SupplementInfoDto infoDto, List<MultipartFile> fileIds) {
    boolean flag;
    List<StocksSupplementDetail> supplementDetails = infoDto.getSupplementDetails();
    if (CollectionUtils.isEmpty(supplementDetails)) {
      throw new RuntimeException("增补出库明细单数据不能为空");
    }
    String baseNo = "ZBCK";
    // 创建一个Date对象，获取当前时间
    Date now = new Date();
    String strDateFormat = "yyyyMM";
    String startNo = baseNo + DateFormatUtils.format(now, strDateFormat);
    // 获取当天最大的采样编号
    String sampleNo = this.baseMapper.getSupplementMaxNo(startNo);

    int firstNum = 1;
    if (sampleNo.length() == 14) {
      firstNum = Integer.parseInt(sampleNo.substring(11)) + 1;
    }
    // 补全数字，不足用0补位
    String endNo = String.format("%04d", firstNum);

    String supplementNo = startNo + endNo;

    StocksSupplementInfo data = new StocksSupplementInfo();
    BeanUtils.copyProperties(infoDto, data);
    // 赋值单号
    data.setSupplementCode(supplementNo);
    // 默认入库人为当前登录人
    data.setSupplementUser(SecurityUtils.getUserId());
    data.setSupplementDept(SecurityUtils.getUserDeptId());
    // 附件
    data.setAnnex(
        fileInfoService.uploadFiles(
            fileIds, data.getAnnex(), data.getAnnex(), path, "TF_STOCKS_OUTSTOCK_INFO"));

    flag = this.save(data);
    // 保存入库明细数据
    for (StocksSupplementDetail supplementDetail : supplementDetails) {
      supplementDetail.setOutstockNum(baseMapper.getOutStockNum(data.getOutStockCode(),supplementDetail.getStockInfoId()));
      supplementDetail.setSupplementInfoId(data.getId());
      stocksSupplementDetailService.save(supplementDetail);
    }
    handleStockInfo(supplementDetails, infoDto.getSupplementType());
    return flag;
  }

  /**
   * 修改增补出库信息
   *
   * @param infoDto dto
   * @param fileIds 文件
   * @return boolean
   */
  @Override
  @Transactional(rollbackFor = Exception.class)
  public boolean editSupplementInfo(SupplementInfoDto infoDto, List<MultipartFile> fileIds) {

    StocksSupplementInfo supplementInfo = this.getById(infoDto.getId());
    if (supplementInfo == null) {
      throw new RuntimeException("修改，参数Id错误");
    }
    if (!supplementInfo.getSupplementUser().equals(SecurityUtils.getUserId())) {
      throw new RuntimeException("只能修改自己创建的数据");
    }
    infoDto.setSupplementCode(supplementInfo.getSupplementCode());
    StocksSupplementInfo data = new StocksSupplementInfo();
    BeanUtils.copyProperties(infoDto, data);

    data.setAnnex(
        fileInfoService.uploadFiles(
            fileIds,
            data.getAnnex(),
            supplementInfo.getAnnex(),
            path,
            "TF_STOCKS_SUPPLEMENT_INFO"));
    this.updateById(data);

    Long type = supplementInfo.getSupplementType();
    // 明细详情
    List<StocksSupplementDetail> detailList =
        stocksSupplementDetailService.list(
            Wrappers.<StocksSupplementDetail>lambdaQuery()
                .eq(StocksSupplementDetail::getSupplementInfoId, supplementInfo.getId()));

    cleanStockInfo(type, detailList);

    for (StocksSupplementDetail supplementDetail : infoDto.getSupplementDetails()) {
      supplementDetail.setSupplementInfoId(data.getId());
      stocksSupplementDetailService.save(supplementDetail);
    }
    handleStockInfo(infoDto.getSupplementDetails(), type);
    return true;
  }

  private void cleanStockInfo(Long type, List<StocksSupplementDetail> detailList) {
    for (StocksSupplementDetail supplementDetail : detailList) {
      StocksStockInfo stockInfo = stocksStockInfoService.getById(supplementDetail.getStockInfoId());

      if (stockInfo == null) {
        throw new ToflyDeniedException("该库存信息不存在");
      }

      if (type.equals(StocksAuditEnum.OUT_STOCK_INSERT_TYPE.getCode())) {
        stockInfo.setOutstockTotalNum(
            stockInfo.getOutstockTotalNum() - supplementDetail.getSupplementNum());
        stockInfo.setStockNum(stockInfo.getStockNum() + supplementDetail.getSupplementNum());
      } else {
        if (stockInfo.getStockNum() < supplementDetail.getSupplementNum()) {
          throw new ToflyDeniedException("库存数量不足");
        }
        stockInfo.setOutstockTotalNum(
            stockInfo.getOutstockTotalNum() - supplementDetail.getSupplementNum());
        stockInfo.setStockNum(stockInfo.getStockNum() + supplementDetail.getSupplementNum());
      }
      stocksStockInfoService.updateById(stockInfo);
    }

    List<Long> detailIdList =
        detailList.stream().map(StocksSupplementDetail::getId).collect(Collectors.toList());
    stocksSupplementDetailService.removeByIds(detailIdList);
  }

  /**
   * 删除增补出库信息
   *
   * @param ids ids
   * @return boolean
   */
  @Override
  @Transactional(rollbackFor = Exception.class)
  public boolean deleteSupplementInfo(String ids) {
    List<String> idList = Arrays.asList(ids.split(","));

    for (String id : idList) {
      StocksSupplementInfo info = getById(id);
      Long type = info.getSupplementType();
      // 明细详情
      List<StocksSupplementDetail> detailList =
          stocksSupplementDetailService.list(
              Wrappers.<StocksSupplementDetail>lambdaQuery()
                  .eq(StocksSupplementDetail::getSupplementInfoId, id));

      cleanStockInfo(type, detailList);
    }

    return removeByIds(idList);
  }

  @Override
  public Page<OutstockInfoVo> getDataPage(Page page, SupplementInfoQuery query) {
    String input = query.getInput();
    Date endDate = query.getEndDate();
    Date startDate = query.getStartDate();
    Long firm = query.getFirm();
    Long type = query.getType();

    return page(
        page,
        Wrappers.<StocksSupplementInfo>lambdaQuery()
            .eq(type != null, StocksSupplementInfo::getSupplementType, type)
            .and(StringUtils.isNotBlank(input),w->w.like(StocksSupplementInfo::getSupplementCode, input)
                    .or().like(StocksSupplementInfo::getOutStockCode,input))
            .eq(firm != null, StocksSupplementInfo::getFirm, firm)
            .between(
                startDate != null && endDate != null,
                StocksSupplementInfo::getSupplementDate,
                startDate,
                endDate));
  }

  /**
   * 查询当前出库单退库数量
   *
   * @param code 出库单编码
   * @return list
   */
  @Override
  public List<Map<String, Object>> queryReturnNumByCode(String code) {
    return baseMapper.queryReturnNumByCode(code);
  }

  private void handleStockInfo(List<StocksSupplementDetail> supplementDetails, Long type) {
    for (StocksSupplementDetail supplementDetail : supplementDetails) {
      StocksStockInfo stockInfo = stocksStockInfoService.getById(supplementDetail.getStockInfoId());

      if (stockInfo == null) {
        throw new ToflyDeniedException("该库存信息不存在");
      }

      if (type.equals(StocksAuditEnum.OUT_STOCK_INSERT_TYPE.getCode())) {
        if (stockInfo.getStockNum() < supplementDetail.getSupplementNum()) {
          throw new ToflyDeniedException("该库存数量不足,请检查库存");
        }
        stockInfo.setOutstockTotalNum(
            stockInfo.getOutstockTotalNum() + supplementDetail.getSupplementNum());
        stockInfo.setStockNum(stockInfo.getStockNum() - supplementDetail.getSupplementNum());
      } else {
        stockInfo.setOutstockTotalNum(
            stockInfo.getOutstockTotalNum() - supplementDetail.getSupplementNum());
        stockInfo.setStockNum(stockInfo.getStockNum() + supplementDetail.getSupplementNum());
      }
      stocksStockInfoService.updateById(stockInfo);
    }
  }


}
