package com.tofly.yswzMaterials.service.impl;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.yswzMaterials.em.StocksAuditEnum;
import com.tofly.yswzMaterials.entity.StocksAudit;
import com.tofly.yswzMaterials.entity.StocksTakeawayDetail;
import com.tofly.yswzMaterials.entity.StocksTakeawayInfo;
import com.tofly.yswzMaterials.entity.dto.StocksTakeawayInfoDto;
import com.tofly.yswzMaterials.mapper.StocksTakeawayInfoMapper;
import com.tofly.yswzMaterials.service.IFileInfoService;
import com.tofly.yswzMaterials.service.StocksAuditService;
import com.tofly.yswzMaterials.service.StocksTakeawayDetailService;
import com.tofly.yswzMaterials.service.StocksTakeawayInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author bubble
 */
@Service
public class StocksTakeawayInfoServiceImpl
    extends ServiceImpl<StocksTakeawayInfoMapper, StocksTakeawayInfo>
    implements StocksTakeawayInfoService {
  @Autowired private IFileInfoService fileInfoService;
//  @Autowired private StocksAuditService stocksAuditService;
  @Autowired private StocksTakeawayDetailService stocksTakeawayDetailService;

  @Value("${yswz.ftp.takeawayInfo}")
  private String path;

  /**
   * 保存外卖出库信息
   *
   * @param info 出库信息
   * @param file 文件
   * @return Boolean.TRUE
   */
  @Override
  @Transactional(rollbackFor = Exception.class)
  public boolean saveTakeawayInfo(StocksTakeawayInfo info, List<MultipartFile> file) {
    List<StocksTakeawayDetail> detailList = info.getDetailList();
    if (CollectionUtils.isNotEmpty(detailList)){
      throw new RuntimeException("请选择物料");
    }
    if (CollectionUtils.isNotEmpty(file)){
      info.setAnnex(
              fileInfoService.uploadFiles(
                      file, info.getAnnex(), info.getAnnex(), path, "TF_STOCKS_OUTSTOCK_INFO"));
    }
    // 默认入库人为当前登录人
    info.setTakeawayUser(SecurityUtils.getUserId());
    info.setTakeawayDept(SecurityUtils.getUserDeptId());
    info.setAuditStatus(StocksAuditEnum.NOT_REVIEWED.getCode());
    save(info);
    detailList.forEach(detail->{
      detail.setTakeawayId(info.getId());
      stocksTakeawayDetailService.save(detail);
    });
    return true;
  }

  /**
   * 通过id批量删除外卖出库信息
   *
   * @param ids id
   * @return Boolean.TRUE
   */
  @Override
  public boolean removeByIds(String ids) {
    if (StringUtils.isBlank(ids)) {
        throw new ToflyDeniedException("未选择外卖出库记录");
      }
      List<String> idList = Arrays.asList(ids.split(","));
      // 查询出库记录
      List<StocksTakeawayInfo> infoList =
          list(
              Wrappers.<StocksTakeawayInfo>lambdaQuery()
                  .in(StocksTakeawayInfo::getId, idList)
                  .select(
                      StocksTakeawayInfo::getId,
                      StocksTakeawayInfo::getBillNo,
                      StocksTakeawayInfo::getAuditStatus));
      // 是否存在未审核以外的记录
      String msg =
          infoList.stream()
              .filter(
                  info -> !info.getAuditStatus().equals(StocksAuditEnum.NOT_REVIEWED.getCode()))
              .map(StocksTakeawayInfo::getBillNo)
              .collect(Collectors.joining(","));
      if (!StringUtils.isEmpty(msg)) {
        throw new ToflyDeniedException(msg + "并未处于未审核接口不可删除");
      }
    //清除审核记录
/*      stocksAuditService.remove(
          Wrappers.<StocksAudit>lambdaQuery()
              .eq(StocksAudit::getAuditType, StocksAuditEnum.TAKEAWAY_OUT_STOCK_TYPE.getCode())
              .in(StocksAudit::getTableId, idList));*/
      // 删除记录
      removeByIds(idList);
      // 删除详情表
      stocksTakeawayDetailService.remove(
          Wrappers.<StocksTakeawayDetail>lambdaQuery()
              .in(StocksTakeawayDetail::getTakeawayId, idList));
    return true;
  }

  /**
   * 编辑外卖出库信息
   *
   * @param info 外卖出库信息
   * @param fileIds 文件
   * @return Boolean.TRUE
   */
  @Override
  @Transactional(rollbackFor = Exception.class)
  public boolean editTakeawayStockInfo(StocksTakeawayInfo info, List<MultipartFile> fileIds) {

    Long takeawayStatus = info.getAuditStatus();
    if (!takeawayStatus.equals(StocksAuditEnum.NOT_REVIEWED.getCode())
        && !takeawayStatus.equals(StocksAuditEnum.HSIANG_HSIEN.getCode())) {
      throw new ToflyDeniedException("该出库单正在审核或审核完成，无法修改");
    }
    if (CollectionUtils.isNotEmpty(fileIds)){
      info.setAnnex(
              fileInfoService.uploadFiles(
                      fileIds, info.getAnnex(), info.getAnnex(), path, "TF_STOCKS_TAKEAWAY_INFO"));
    }
    this.updateById(info);
    stocksTakeawayDetailService.updateBatchById(info.getDetailList());
    return true;
  }
}
