package com.tofly.yswzMaterials.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.yswzMaterials.custom.service.CallApiService;
import com.tofly.yswzMaterials.custom.service.CommonUtilService;
import com.tofly.yswzMaterials.em.JinDieUrlEnum;
import com.tofly.yswzMaterials.em.StocksAuditEnum;
import com.tofly.yswzMaterials.entity.Loss;
import com.tofly.yswzMaterials.entity.StocksAudit;
import com.tofly.yswzMaterials.entity.Storage;
import com.tofly.yswzMaterials.entity.StorageDetail;
import com.tofly.yswzMaterials.entity.dto.CommonQueryDto;
import com.tofly.yswzMaterials.entity.vo.StorageParamVo;
import com.tofly.yswzMaterials.mapper.StorageMapper;
import com.tofly.yswzMaterials.service.IFileInfoService;
import com.tofly.yswzMaterials.service.StocksAuditService;
import com.tofly.yswzMaterials.service.StorageDetailService;
import com.tofly.yswzMaterials.service.StorageService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author Bubble
 */
@Service
public class StorageServiceImpl extends ServiceImpl<StorageMapper, Storage> implements StorageService {
    @Autowired
    private  StorageDetailService storageDetailService;
    @Autowired
    private  CallApiService callApiService;
    @Autowired
    private IFileInfoService fileInfoService;
    @Autowired
    private CommonUtilService commonUtilService;

    @Value("${yswz.ftp.stocksInstockInfo}")
    private String path;

    /**
     * 添加外购入库
     *
     * @param storage storage
     * @return boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean insertStorage(Storage storage, List<MultipartFile> file) {
        Long storageType = storage.getStorageType();
        storage.setBillNo(commonUtilService.getBillNo(storageType == 0 ? 1 : 3, Math.toIntExact(storageType)));
        List<StorageDetail> detailList = storage.getDetailList();
        //登记时为未审核
        storage.setAuditStatus(StocksAuditEnum.NOT_REVIEWED.getCode());
        if (CollectionUtils.isNotEmpty(file)){
            storage.setAnnex(
                    fileInfoService.uploadFiles(
                            file, storage.getAnnex(), storage.getAnnex(), path, "TF_JINDIE_STORAGE"));
        }
        //添加入库单
        save(storage);
        //添加入库单详情
        detailList.forEach(detail->{
            detail.setStorageId(storage.getId());
            storageDetailService.save(detail);
        });
        return true;
    }

    /**
     * 修改外购入库
     *
     * @param storage storage
     * @param file    文件
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateStorage(Storage storage, List<MultipartFile> file) {
        Storage byId = getById(storage.getId());
        Long auditStatus = storage.getAuditStatus();
        if (!auditStatus.equals(StocksAuditEnum.NOT_REVIEWED.getCode())
                && !auditStatus.equals(StocksAuditEnum.HSIANG_HSIEN.getCode())) {
            throw new ToflyDeniedException("非未审核与驳回无法修改");
        }
        if (CollectionUtils.isNotEmpty(file)){
            storage.setAnnex(
                    fileInfoService.uploadFiles(
                            file, storage.getAnnex(),byId.getAnnex(), path, "TF_JINDIE_STORAGE"));
        }

        updateById(storage);
        storageDetailService.updateBatchById(storage.getDetailList());
        return true;
    }

    /**
     * 获取外购入库详情
     *
     * @param code code
     * @return StorageParamVo
     */
    @Override
    public StorageParamVo getStorageInfo(String code) {
        String tokenInfo = callApiService.getToken();
        String url = JinDieUrlEnum.GET_STORAGE_URL.getName() + "?token=" + tokenInfo + "&json=" + code;
        Map apiResult = callApiService.getApiResult(null, url);
        if (CollectionUtils.isNotEmpty(apiResult)){
            return JSONObject.parseObject(String.valueOf(apiResult.get("Message")), StorageParamVo.class);
        }
        return null;
    }

    /**
     * 分页查询
     *
     * @param page page
     * @param dto  dto
     * @return page
     */
    @Override
    public Page<Storage> page(Page page, CommonQueryDto dto) {
        Long auditStatus = dto.getAuditStatus();
        Date startTime = dto.getStartTime();
        Date endTime = dto.getEndTime();
        String input = dto.getInput();
        Long type = dto.getType();
        return page(page, Wrappers.<Storage>lambdaQuery()
                .like(StringUtils.isNotBlank(input),Storage::getBillNo, input)
                .eq(type != null, Storage::getStorageType, type)
                .eq(auditStatus != null, Storage::getAuditStatus, auditStatus)
                .between(startTime != null && endTime != null,Storage::getFDate, startTime, endTime));
    }

    /**
     * 物资入库删除
     *
     * @param ids ids
     * @return boolean
     */
    @Override
    public boolean delete(String ids) {
        List<String> idList = Arrays.asList(ids.split(","));
        List<Storage> storageList = list(Wrappers.<Storage>lambdaQuery().in(Storage::getId, idList));
        if (CollectionUtils.isNotEmpty(storageList)){
            long count = storageList.stream().filter(s -> !s.getAuditStatus().equals(StocksAuditEnum.NOT_REVIEWED.getCode())
                    && !s.getAuditStatus().equals(StocksAuditEnum.HSIANG_HSIEN.getCode())).count();
            if (count > 0){
                throw new ToflyDeniedException("非未审核或驳回入库单，无法删除");
            }
        }
        removeByIds(idList);
        storageDetailService.remove(Wrappers.<StorageDetail>lambdaQuery()
                .in(StorageDetail::getStorageId, idList));
        return true;
    }
}
