package com.tofly.yswzMaterials.service.jindie.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.yswzMaterials.custom.service.CallApiService;
import com.tofly.yswzMaterials.em.JinDieUrlEnum;
import com.tofly.yswzMaterials.entity.*;
import com.tofly.yswzMaterials.entity.jindie.Material;
import com.tofly.yswzMaterials.mapper.StocksAuditMapper;
import com.tofly.yswzMaterials.service.*;
import com.tofly.yswzMaterials.service.jindie.DockingService;
import lombok.AllArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 金蝶对接接口
 *
 * @author Bubble
 */
@Service
@AllArgsConstructor
public class DockingServiceImpl implements DockingService {

  private final CallApiService callApiService;
  private final StocksStoreService stocksStoreService;
  private final StocksVendorService stocksVendorsService;
  private final StocksDepartmentService stocksDepartmentService;
  private final StocksEmployeeService stocksEmployeeService;
  private final StocksUnitService stocksUnitService;
  private final StocksDeviceInfoService stocksDeviceInfoService;
  private final StocksAuditMapper stocksAuditMapper;

  @Override
  public List<Material> basicInfoQueryAll(String urlCode) {
    String token = callApiService.getToken();
    if (StringUtils.isBlank(token)) {
      throw new ToflyDeniedException("token获取失败");
    }
    String url;
    switch (urlCode) {
      case "0":
        url = JinDieUrlEnum.GET_ALL_MATERIAL_URL.getName();
        break;
      case "1":
        url = JinDieUrlEnum.GET_ALL_WAREHOUSE_URL.getName();
        break;
      case "2":
        url = JinDieUrlEnum.GET_ALL_VENDOR_URL.getName();
        break;
      case "3":
        url = JinDieUrlEnum.GET_ALL_DEPARTMENT_URL.getName();
        break;
      case "4":
        url = JinDieUrlEnum.GET_ALL_EMPLOYEE_TYPE.getName();
        break;
      case "5":
        url = JinDieUrlEnum.GET_ALL_UNIT_URL.getName();
        break;
      default:
        url = "";
    }
    Map apiResult =
        callApiService.getApiResult(null, url + "?token=" + callApiService.getToken() + "&json=%25");
    return JSONObject.parseArray(String.valueOf(apiResult.get("Message")), Material.class);
  }

  /**
   * 基础数据查询接口
   *
   * @param material material
   * @param urlCode urlCode
   * @return list
   */
  @Override
  @Transactional(rollbackFor = Exception.class)
  public Map basicInfoQuery(Material material, String urlCode, String type) {

    Map<String, Object> body = new HashMap<>(1);
    if (StringUtils.isNotBlank(material.getFNumber())) {
      body.put("FNumber", material.getFNumber());
    }
    if (StringUtils.isNotBlank(material.getFName())) {
      body.put("FName", material.getFName());
    }
    if (StringUtils.isNotBlank(material.getFModel())) {
      body.put("FModel", material.getFModel());
    }
    if (StringUtils.isNotBlank(material.getFUnitGroup())) {
      body.put("FUnitGroup", material.getFUnitGroup());
    }
    if (StringUtils.isNotBlank(material.getFUnit())) {
      body.put("FUnit", material.getFUnit());
    }
    if (StringUtils.isNotBlank(material.getFTaxRate())) {
      body.put("FTaxRate", material.getFTaxRate());
    }
    if (StringUtils.isNotBlank(material.getFCoefficient())) {
      body.put("FCoefficient", material.getFCoefficient());
    }

    String token = callApiService.getToken();
    if (StringUtils.isBlank(token)) {
      throw new ToflyDeniedException("token获取失败");
    }
    String url = null;
    switch (urlCode) {
      case "0":
        if ("0".equals(type)) {
          url = JinDieUrlEnum.GET_MATERIAL_URL.getName();
        } else if ("1".equals(type)) {
          //新增物料
          url = JinDieUrlEnum.SAVE_MATERIAL_URL.getName();
          stocksDeviceInfoService.save(JSONObject.parseObject(setEntityValue(material,"0"), StocksDeviceInfo.class));
        } else if ("2".equals(type)) {
          //删除物料
          url = JinDieUrlEnum.DELETE_MATERIAL_URL.getName();
          stocksDeviceInfoService.remove(Wrappers.<StocksDeviceInfo>lambdaQuery().eq(StocksDeviceInfo::getDeviceNo, material.getFNumber()));
        }

        break;
      case "1":
        if ("0".equals(type)) {
          url = JinDieUrlEnum.GET_WAREHOUSE_URL.getName();
        } else if ("1".equals(type)) {
          url = JinDieUrlEnum.SAVE_WAREHOUSE_URL.getName();
          stocksStoreService.save(JSONObject.parseObject(setEntityValue(material,"1"), StocksStore.class));
        } else if ("2".equals(type)) {
          url = JinDieUrlEnum.DELETE_WAREHOUSE_URL.getName();
          stocksStoreService.remove(Wrappers.<StocksStore>lambdaQuery().eq(StocksStore::getStoreNumber, material.getFNumber()));
        }
        break;
      case "2":
        if ("0".equals(type)) {
          url = JinDieUrlEnum.GET_VENDOR_URL.getName();
        } else if ("1".equals(type)) {
          url = JinDieUrlEnum.SAVE_VENDOR_URL.getName();
          StocksVendor vendor = new StocksVendor();
          vendor.setFname(material.getFName());
          vendor.setFnumber(material.getFNumber());
          stocksVendorsService.save(vendor);
        } else if ("2".equals(type)) {
          url = JinDieUrlEnum.DELETE_VENDOR_URL.getName();
          stocksVendorsService.remove(Wrappers.<StocksVendor>lambdaQuery().eq(StocksVendor::getFnumber, material.getFNumber()));
        }
        break;
      case "3":
        if ("0".equals(type)) {
          url = JinDieUrlEnum.GET_DEPARTMENT_URL.getName();
        } else if ("1".equals(type)) {
          url = JinDieUrlEnum.SAVE_DEPARTMENT_URL.getName();
          StocksDepartment department = new StocksDepartment();
          department.setFnumber(material.getFNumber());
          department.setFname(material.getFName());
          stocksDepartmentService.save(department);
        } else if ("2".equals(type)) {
          url = JinDieUrlEnum.DELETE_DEPARTMENT_URL.getName();
          stocksDepartmentService.remove(Wrappers.<StocksDepartment>lambdaQuery().eq(StocksDepartment::getFnumber, material.getFNumber()));
        }
        break;
      case "4":
        if ("0".equals(type)) {
          url = JinDieUrlEnum.GET_EMPLOYEE_TYPE.getName();
        } else if ("1".equals(type)) {
          url = JinDieUrlEnum.SAVE_EMPLOYEE_TYPE.getName();
          StocksEmployee employee = new StocksEmployee();
          employee.setFnumber(material.getFNumber());
          employee.setFname(material.getFName());
          stocksEmployeeService.save(employee);
        } else if ("2".equals(type)) {
          url = JinDieUrlEnum.DELETE_EMPLOYEE_TYPE.getName();
          stocksEmployeeService.remove(Wrappers.<StocksEmployee>lambdaQuery().eq(StocksEmployee::getFnumber, material.getFNumber()));
        }
        break;
      case "5":
        if ("0".equals(type)) {
          url = JinDieUrlEnum.GET_UNIT_URL.getName();
        } else if ("1".equals(type)) {
          url = JinDieUrlEnum.SAVE_UNIT_URL.getName();
          StocksUnit unit = new StocksUnit();
          unit.setFnumber(material.getFNumber());
          unit.setFname(material.getFName());
          unit.setFunitgroup(material.getFUnitGroup());
          unit.setFcoefficient(material.getFCoefficient());
          stocksUnitService.save(unit);
        } else if ("2".equals(type)) {
          url = JinDieUrlEnum.DELETE_UNIT_URL.getName();
          stocksUnitService.remove(Wrappers.<StocksUnit>lambdaQuery().eq(StocksUnit::getFnumber, material.getFNumber()));
        }
        break;
      default:
        url = "";
    }
    return callApiService.getApiResult(
        "1".equals(type) ? body : material.getFNumber(),
        url + "?token=" + callApiService.getToken());
  }

  /**
   * 同步基础信息
   * @param type 0 物资 1 仓库 2 供应商 3 部门 4 员工 5 计量单位
   * @return boolean
   */
  @Override
  @Transactional(rollbackFor = Exception.class)
  public boolean syncBasicInfo(String type) {
    List<Material> materialList = basicInfoQueryAll(type);
    if (CollectionUtils.isNotEmpty(materialList)) {
      //编码
      Set<String> numberSet = materialList.stream().filter(Objects::nonNull)
              .map(Material::getFNumber).collect(Collectors.toSet());
      switch (type) {
        case "0":
          //清空物料
          stocksDeviceInfoService.deleteInfoByNumber(numberSet);
          List<StocksDeviceInfo> info = new ArrayList<>();

          Long maxId = stocksAuditMapper.getMaxId("TF_STOCKS_DEVICE_INFO");
          for (Material material : materialList) {
            StocksDeviceInfo deviceInfo = JSONObject.parseObject(setEntityValue(material, "0"), StocksDeviceInfo.class);
            deviceInfo.setId(maxId++);
            info.add(deviceInfo);
          }
          stocksDeviceInfoService.batchInsertInfo(info);
          break;
        case "1":
          //仓库数据同步
          // 首先清除仓库编码存在的数据
          stocksStoreService.remove(Wrappers.<StocksStore>lambdaQuery()
                  .in(StocksStore::getStoreNumber, numberSet));
          for (Material material : materialList) {
            stocksStoreService.save(JSONObject.parseObject(setEntityValue(material,"1"), StocksStore.class));
          }
          break;
        case "2":
          stocksVendorsService.remove(Wrappers.<StocksVendor>lambdaQuery()
                  .in(StocksVendor::getFnumber, numberSet));
          stocksVendorsService.saveBatch(JSONObject.parseArray(JSONObject.toJSONString(materialList), StocksVendor.class));
          break;
        case "3":
          stocksDepartmentService.remove(Wrappers.<StocksDepartment>lambdaQuery()
                  .in(StocksDepartment::getFnumber, numberSet));
          stocksDepartmentService.saveBatch(JSONObject.parseArray(JSONObject.toJSONString(materialList), StocksDepartment.class));
          break;
        case "4":
          stocksEmployeeService.remove(Wrappers.<StocksEmployee>lambdaQuery()
                  .in(StocksEmployee::getFnumber, numberSet));
          stocksEmployeeService.saveBatch(JSONObject.parseArray(JSONObject.toJSONString(materialList), StocksEmployee.class));
          break;
        case "5":
          stocksUnitService.remove(Wrappers.<StocksUnit>lambdaQuery()
                  .in(StocksUnit::getFnumber, numberSet));
          stocksUnitService.saveBatch(JSONObject.parseArray(JSONObject.toJSONString(materialList), StocksUnit.class));
          break;
        default:
      }
    }
    return true;
  }

  /**
   * 获取物料单位组
   * @return list
   */
  @Override
  public List<String> getDeviceUnitGroup() {
    return stocksDeviceInfoService.getDeviceUnitGroup();
  }

  /**
   * 获取厂商列表
   *
   * @return list
   */
  @Override
  public Page getVendorList(Page page) {
    return stocksVendorsService.page(page);
  }

  /**
   * 获取部门列表
   *
   * @param page page
   * @return page
   */
  @Override
  public Page getDeptList(Page page) {
    return stocksDepartmentService.page(page);
  }

  /**
   * 获取金蝶人员列表
   *
   * @param page page
   * @return page
   */
  @Override
  public Page getEmployeeList(Page page) {
    return stocksEmployeeService.page(page);
  }

  private String setEntityValue(Material material,String type){
    Long userId = SecurityUtils.getUserId();
    if ("0".equals(type)){
      StocksDeviceInfo deviceInfo = new StocksDeviceInfo();
      deviceInfo.setDeviceNo(material.getFNumber());
      deviceInfo.setDeviceName(material.getFName());
      deviceInfo.setDeviceSpecsModel(material.getFModel());
      deviceInfo.setUnitGroup(material.getFUnitGroup());
      deviceInfo.setTaxRate(material.getFTaxRate());
      deviceInfo.setDeviceUnitName(material.getFUnit());
      deviceInfo.setVendor(material.getVendor());
      deviceInfo.setWarnInventory(material.getWarnInventory());
      deviceInfo.setLotNumber(material.getLotNumber());
      return JSONObject.toJSONString(deviceInfo);
    }else {
      StocksStore store = new StocksStore();
      store.setStoreName(material.getFName());
      store.setStoreNumber(material.getFNumber());
      //管理员
      store.setStoreManagerUserId(userId);
      store.setCreateUser(userId);
      store.setStoreFunction(material.getStoreFunction());
      store.setCreateTime(new Date());
      //所属公司 营山县圣新天然气有限公司
      store.setFirm(17L);
      store.setStoreEnable("1");
      return JSONObject.toJSONString(store);
    }
  }
}
